/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QObject>
#include <QFileInfo>
#include <QDir>
#include <QSettings>
#include <QMessageBox>

#ifdef Q_WS_X11
#include <QProcess>
//#include <plasma/wallpaper.h>
#endif

#ifdef Q_WS_WIN
  #include <fstream>
  #include <windows.h>
#endif

#ifdef Q_WS_PM
  #include <os2.h>
  #include <os2emx.h>
#endif

#include <QDebug>
#include "mainwindow.h"
#include "backgroudimage.h"
#include "network.h"


QString backgroudImage::windowManager="";

backgroudImage::backgroudImage(const QString &url )
{
    QByteArray b;
    QPixmap pix;
    QImage img;

    if (windowManager=="") linuxDetectWM();
    if (windowManager=="") {
        QMessageBox::critical(0,QObject::tr("error"),QObject::tr("Window manager not detected") );
        return;
    }
    if (windowManager=="KDE4")
    {
//        TODO
//        dunno how to change wallpaper here
        QMessageBox::critical(0,QObject::tr("error"),QObject::tr("Can't set wallpaper on Kde4 yet") );
        return;
    }

#ifdef Q_WS_PM
    // remove background
    HOBJECT hobj;
    const char *objName = "<WP_DESKTOP>";

    if (hobj = WinQueryObject(reinterpret_cast< const unsigned char * > (objName)))
        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > ((QString("BACKGROUND=\"\",N,1,I;")).toAscii().constData()));
#endif

    network *net_async=new network(false, MainWindow::pointerToMainWindow);
    b=net_async->get_synchronous(url,false);
    if (!b.isEmpty())
    {
        pix.loadFromData(b);
        img=pix.toImage();
    }
    net_async->deleteLater();
    net_async=0;

    if (!b.isEmpty())
        setBackground(img);
    else
        QMessageBox::critical(0,QObject::tr("download wallpaper error"),QObject::tr("Can't download: ")+url,QMessageBox::Ok );
}

void backgroudImage::setBackground(QImage &image)
{
    QString imageCompletePath;
    QStringList args;

#ifdef Q_WS_X11
    if (windowManager=="GNOME")
    {
        imageCompletePath=QDir::homePath()+"/MDwallpaper.jpg";
        image.save(imageCompletePath,"JPG");
        args << "--type" << "bool" << "--set" << "/desktop/gnome/background/draw_background" << "true";
        QProcess::execute("gconftool-2",args);
        args.clear();
        args << "--type" << "string" << "--set" << "/desktop/gnome/background/picture_options" << "scaled";
        QProcess::execute("gconftool-2",args);
        args.clear();
        args << "--type" << "string" << "--set" <<
                "/desktop/gnome/background/picture_filename" << imageCompletePath;
        QProcess::execute("gconftool-2",args);
    }

    if (windowManager=="KDE4")
    {
//        TODO
//        dunno how to change wallpaper here
    }

    if (windowManager=="KDE3")
    {
        imageCompletePath=QDir::homePath()+"/MDwallpaper.jpg";
        image.save(imageCompletePath,"JPG");
        args << "kdesktop" << "KBackgroundIface" << "setWallpaper"
                << imageCompletePath << "1";
        QProcess::execute("dcop",args);
    }
#endif

#ifdef Q_WS_WIN
    imageCompletePath=QDir::homePath()+"/MDwallpaper.bmp";
    image.save(imageCompletePath,"BMP");

    QSettings regEdit("HKEY_CURRENT_USER\\Control Panel\\Desktop",QSettings::NativeFormat);
    regEdit.setValue("Wallpaper",QDir::toNativeSeparators(imageCompletePath));
    regEdit.setValue("WallpaperStyle","0");
    regEdit.setValue("TileWallpaper","0");
    QString s = QDir::toNativeSeparators(imageCompletePath.toAscii());
    bool b=SystemParametersInfoA(SPI_SETDESKWALLPAPER,0,
                          reinterpret_cast<void *>
                          (QDir::toNativeSeparators(imageCompletePath).toAscii().data()),
                          SPIF_SENDCHANGE);//SPIF_SENDWININICHANGE);
    qDebug() << "Wallpaper updated:" << b;
#endif

#ifdef Q_WS_PM
    imageCompletePath=QDir::homePath()+"/MDwallpaper.bmp";
    QFile::remove(imageCompletePath);
    image.save(imageCompletePath,"BMP");

    HOBJECT hobj;
    const char *objName = "<WP_DESKTOP>";

    if (hobj = WinQueryObject(reinterpret_cast< const unsigned char * > (objName)))
//        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > ((QString().arg(QDir::toNativeSeparators(imageCompletePath))).toAscii().constData()));
        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > (QDir::toNativeSeparators(imageCompletePath).toAscii().constData()));
#endif
}


QString backgroudImage::linuxDetectWM()
{
#ifdef Q_WS_X11
    QList<QFileInfo> files = QDir("/proc").entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);
    QFileInfo file;
    int i;
    QString s;

    i=0;
    while (i<files.size())
    {
        if (QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit") &&
            !QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("start_kdeinit") &&
            !QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit4"))
        {
            s=QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
            windowManager="KDE3";
            break;
        }
        if (QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit4"))
        {
            s=QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
            windowManager="KDE4";
            break;
        }
        if (QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("gconfd-2"))
        { windowManager="GNOME"; break; }
        qDebug() << QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
        i++;
    }
#endif

#ifdef Q_WS_WIN
    windowManager="WINDOWS";
#endif

#ifdef Q_WS_MAC
    windowManager="MAC";
#endif

#ifdef Q_WS_PM
    windowManager="OS/2";
#endif

    qDebug() << "Windows manager detected: " << windowManager;
    return windowManager;
}
