/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef FIREFOXMONITOR_H
#define FIREFOXMONITOR_H

#include <QObject>
#include <QDir>
#include <QFile>
#include <QFileSystemWatcher>
#include <QTimer>
#include "imageviewer2d.h"


class firefoxMonitor : public QObject
{
    Q_OBJECT
public:
    typedef struct {
        quint32 magic;
        quint32 location;
        quint32 fetchcount;
        quint32 fetchtime;
        quint32 modifytime;
        quint32 expiretime;
        quint32 datasize;
        quint32 requestsize;
        quint32 infosize;
        QByteArray requestString;
        QByteArray infoString;
    } cache_data;

    explicit firefoxMonitor(ImageViewer2d *viewer, bool *canMonitor, QObject *parent = 0);
    ~firefoxMonitor();

private:
    void readCache();
    void addRecord(cache_data record);
    bool getFirefoxCacheDir();
    bool readFirefoxUrl();
    void addYouTubeByUrl(QString url);

    QFileSystemWatcher cacheWatcher;
    quint32 magic;
    quint32 now;
    ImageViewer2d *scene;
    imageStruct img;
    QTimer timer;
    QDir firefoxCacheDir;
    QDir firefoxDir;
    QDir mediadownloaderDir;

public slots:
    void onFirefoxCacheChanged(QString file);
    void onTimeElapsed();

signals:
    void outMessage(const QString &message, int row);

};

#endif // FIREFOXMONITOR_H
