/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QString>
#include <QImage>
#include <QBuffer>
#include <QImageReader>
#include "fromfs.h"
#include "defines.h"

fromfs::fromfs(bool subDir, ImageViewer2d *v, QWidget *parent) :
    QWidget(parent)
{
    includeSubDir=subDir;
    viewer=v;

    foreach (QByteArray l, QImageReader::supportedImageFormats())
        mediaImageExtensions << "*."+l;
    mediaVideoExtensions <<
            "*.avi" <<
            "*.dvx" <<
            "*.divx" <<
            "*.mkv" <<
            "*.xvid" <<
            "*.flv" <<
            "*.mp4" <<
            "*.mpeg" <<
            "*.mpg";
}

void fromfs::parseFiles(QDir directory)
{
    int start_nwidget;

    directory.setNameFilters(mediaImageExtensions + mediaVideoExtensions);
    directory.setFilter(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Files);
    directory.setSorting(QDir::Name | QDir::DirsFirst | QDir::IgnoreCase);

    QByteArray thumb;
    QFileInfoList list;
    int n_video,n_images;
    imageStruct *image=0;
    imageContainerKind kind;
    list = directory.entryInfoList();
    start_nwidget = viewer->itemsList.size()-1;

    QString s;
    n_video=0;
    n_images=0;
    foreach (QFileInfo f, list) {
        if (f.isDir() && includeSubDir)
            parseFiles(QDir(f.absoluteFilePath()));
        thumb.clear();

        if (image!=0) delete image;
        image = new imageStruct;

        if (mediaImageExtensions.contains("*."+f.suffix(), Qt::CaseInsensitive))
        {
            getThumbImageThumb(f.absoluteFilePath(), &thumb, image);
            kind=CONTAINER_KIND_IMAGE_LOCAL;
            s=tr("Images: ")+QString::number(n_images);
            emit outMessage(s,1);
            n_images++;
        }
#ifdef PHONON_LIB
        if (mediaVideoExtensions.contains("*."+f.suffix(), Qt::CaseInsensitive))
        {
            getThumbVideoThumb(f.absoluteFilePath(), &thumb, image);
            kind=CONTAINER_KIND_VIDEO_LOCAL;
            s=tr("Videos: ")+QString::number(n_video);
            emit outMessage(s,2);
            n_video++;
        }
#endif
        if (!thumb.isEmpty()) {
            image->image_url = f.absoluteFilePath();
            image->ID = start_nwidget + n_video + n_images;
            image->kind=kind;
            image->description_row1 = f.fileName();
            image->description_row2 += " - "+QString::number(f.size()/1024)+" Kb";
            image->description_row2 = f.suffix().toUpper() + "  " + image->description_row2;
//            viewer->add_imageContainer(thumb, *image);
            viewer->sceneOGL.addItem(viewer->add_imageContainer(thumb, *image));
            viewer->arrange_items(true);
        }
        delete image;
        image=0;

        QApplication::processEvents();
    }
}

void fromfs::getThumbImageThumb(QString absPath, QByteArray *data, imageStruct *is)
 {
    QImage img(absPath);
    QSize s=img.size();
    is->description_row2=QString::number(s.width())+"x"+QString::number(s.height());
    QBuffer buffer(data);
    buffer.open(QIODevice::WriteOnly);
    img.save(&buffer, "JPG");
 }

#ifdef PHONON_LIB
// TODO: add timer for vidoeState loops
void fromfs::getThumbVideoThumb(QString absPath, QByteArray *data, imageStruct *is)
{
    videoState=Phonon::LoadingState;
    Phonon::VideoPlayer *player = new Phonon::VideoPlayer(Phonon::VideoCategory);
    connect(player->mediaObject(), SIGNAL(stateChanged(Phonon::State,Phonon::State)) ,
            this, SLOT(stateChanged(Phonon::State,Phonon::State)) );
    player->setGeometry(0,0, 320, 240);
    QGraphicsScene scene;
    scene.addWidget(player);

    player->play(absPath);
    while (videoState!=Phonon::PlayingState) QApplication::processEvents();

    player->seek(rand()%player->totalTime());
    player->pause();

    while (videoState!=Phonon::PausedState) QApplication::processEvents();

    is->description_row1 = QString::number(player->totalTime())+"\"";

    // Grab frame at a random position
    player->play();
    while (videoState!=Phonon::PlayingState) QApplication::processEvents();

    QPixmap pixmap(player->videoWidget()->size());
    QPainter painter(&pixmap);
    scene.render(&painter);

    player->stop();
    while (videoState!=Phonon::StoppedState) QApplication::processEvents();

    QBuffer buffer(data);
    buffer.open(QIODevice::WriteOnly);
    QImage img=pixmap.toImage();
    // In linux, video played inside graphicsView, rgb are bgr
#ifdef Q_WS_X11
    img=img.rgbSwapped();
#endif
    img.save(&buffer, "JPG");

    delete player;
}

void fromfs::stateChanged(Phonon::State newstate, Phonon::State oldstate)
{
    Q_UNUSED (oldstate);
    videoState=newstate;
}
#endif
