/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef IMAGECONTAINER_H
#define IMAGECONTAINER_H

#include <QObject>
#include <QByteArray>
#include <QGraphicsPixmapItem>
#include <QMenu>
#include <QLabel>
#include "youtube.h"
#include "googlepicture.h"
#include "defines.h"

class imageContainer;

typedef struct {
    int ID;
    imageContainerKind kind;
    GoogleImageStruct googleImageInfo;
    YouTubeStruct youTubeInfo;
    QString description_row1;
    QString description_row2;
    QString description_row3;
    QString image_tburl;
    QString image_url;
    QString image_source;
} imageStruct;

class imageContainer : public QObject , public QGraphicsPixmapItem {
    Q_OBJECT
public:
    enum { REFLEX = UserType + 1 , IMAGE, LABEL, VIDEO, WIDGET }; // UserType = 65536

    imageContainer(QRect r, QPixmap &pixmap);
    imageContainer(QPixmap pix);
    imageContainer(const QByteArray &data, int w, int h, imageStruct img);
    imageContainer(QWidget *w);
    ~imageContainer();

    int type() const
    {
        // Enable the use of qgraphicsitem_cast with this item.
        return Type;
    }

    void change_label_pixmap(QPixmap &pixmap);
    void resize_diapo(QSize size);
    void make_diapo();
    void make_reflex();

    imageStruct image;
    QMenu menu;
    QAction *removeAction;
    QAction *downloadAction;
    QAction *browseAction;
    QAction *setAsBackgroundAction;
    QAction *playWithMplayer;
    QAction *selectedAction;
    uint Type;
    QPixmap p;
    QLabel l;
    QWidget *widget;
    int diapo_width, diapo_heigth;
    static QColor images_color;
    static QColor videos_color;
    static QColor images_text_color;
    static QColor videos_text_color;

private:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void setupMenu();

protected:
    virtual void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event );

private slots:
    void on_browse_clicked();
    void on_remove_clicked();
    void on_download_clicked();
    void on_setAsBackground_clicked();
    void on_playWithMplayer_clicked();

signals:
    void on_menu_remove_clicked(imageContainer *w);
    void on_menu_download_clicked(imageContainer *w);
    void on_menu_browse_clicked(const QString url);
    void on_menu_setAsBackground_clicked(imageContainer *w);
    void double_clicked(imageContainer *w);
    void on_menu_playWithMplayer_clicked(imageContainer *w);
};

#endif // IMAGECONTAINER_H
