/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef IMAGEVIEWER2D_H
#define IMAGEVIEWER2D_H

#include <QApplication>
#include <QGraphicsView>
#include <QGraphicsItem>
#include <QGraphicsWidget>
#include <QGraphicsProxyWidget>
#include <QAbstractScrollArea>

#if QT_VERSION > 0x040600
#include <QGraphicsOpacityEffect>
#include <QPropertyAnimation>
#endif

#ifdef QT_OPENGL_SUPPORT
    #include <QGLWidget>
#endif

#include <QTimer>
#include <QGraphicsItemAnimation>
#include <QTimeLine>
#include <QMap>
#include "imagecontainer.h"
#include "network.h"
#include "mediaplayer.h"

//class OpenGL;
//class ImageViewer2d;

class ImageViewer2d : public QGraphicsView
{
    Q_OBJECT

public:

    ImageViewer2d(QWidget *parent=0);
    ~ImageViewer2d();

    void setImageContainerSize(QSize ic_size, QSize separation);
    void resizeItems(QSize size);
    void add_widget(imageContainer *w);
    imageContainer* add_imageContainer(const QByteArray &data, imageStruct image);
    void addItemList_to_scene();
    void add_all_images(QList<imageStruct> imgs);
    QList<imageStruct> getImageStructList();
    void arrange_items(bool last);
    void zoomItem(imageContainer *from_item, imageContainer *zoom_item);
    void clear_scene();
    void enableContextMenuEvent();
    void disableContextMenuEvent();
    QStringList getImageUrls();
    void set_rows(int n);
    int get_rows();
    void add_row();
    void remove_row();
    void expandWidth();
    void setBackgroundImage(QImage image);
    void setBackgroundGradient(QColor top, QColor mid, QColor bottom);
    virtual void drawBackground(QPainter *painter, const QRectF &f);
    void stopInertia();

    int n_widget;
    static network *net_sync;
    network *net_thumbs;

    QList<imageContainer *> itemsList;
    int widget_height, widget_width;
    int widget_H_separation,widget_V_separation;
    mediaPlayer *media;

    QList<imageStruct> images;
    QColor top_color,mid_color,bottom_color;
    QColor images_color,videos_color,images_text_color,videos_text_color;
    QImage backgroundImage;

    QGraphicsScene sceneOGL;

private:

#ifdef QT_OPENGL_SUPPORT
    QGLWidget *qgl;
#endif
    int start_nwidget;
    int grid_rows;
    qreal zoom_factor;
    qreal DX,DY;
    qreal tot_DY;
    qreal tot_DX;
    qreal inertia_X;
    qreal inertia_Y;
    qreal start_X;
    qreal start_Y;
    int mouse_buttons;
    QTimer timer;
    imageContainer *item_clicked;
    bool contextMenu;
//    bool ctrl_pressed;
    QLinearGradient lg;
    bool checkLimitReached();
#if QT_VERSION > 0x040600
    QGraphicsOpacityEffect *effect;
    QPropertyAnimation *fade;
    QTimeLine *timeLine;
    QGraphicsItemAnimation *animation;
#endif

protected:
//    virtual void dropEvent ( QDropEvent * event );
//    virtual void keyPressEvent ( QKeyEvent * event );
//    virtual void keyReleaseEvent ( QKeyEvent * event );
    virtual void mousePressEvent ( QMouseEvent * );
    virtual void mouseReleaseEvent ( QMouseEvent * );
    virtual void mouseMoveEvent ( QMouseEvent * );
    virtual void contextMenuEvent ( QContextMenuEvent * event );

public slots:
//    void onItemDragging(imageContainer *item);
    void inertia();
    void onItemDoubleClicked(imageContainer *item);
    void onRemoveWidget(imageContainer *w);
    void onItem_Download_clicked(imageContainer *item);
    void onItem_playWithMplayer_clicked(imageContainer *item);
    void onItem_Browse_clicked(const QString &url);
    void onItem_SetAsBackground_clicked(imageContainer*);
    void onDownloaded(int id,const QByteArray &data);
    void onDownloadThumbsCompleted();
    void onMediaItemClosed();

signals:
    void item_double_clicked();
//    void itemDragging(imageContainer *i);
    void item_download_button_clicked(imageContainer *item);
    void item_playWithMplayer_button_clicked(imageContainer *item);
    void item_browse_button_clicked(const QString &url);
    void item_setAsBackground_button_clicked(imageContainer*);
    void networking_finished();
    void outMessage(const QString &message, int row);
    void onScrollRightLimitReached();
};

#endif // IMAGEVIEWER2D_H
