/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QUrl>
#include <QDebug>
#include <QFormLayout>
#include <QFileDialog>
#include <QDir>
#include <QProcess>
#include <QMessageBox>
#include <QDesktopServices>
//  Static building ?
//#if !defined(QT_SHARED) && !defined(QT_DLL) && !defined(FREEIMAGE_LIB)
//#include <QtPlugin>
//Q_IMPORT_PLUGIN(qgif)
//Q_IMPORT_PLUGIN(qjpeg)
//#endif
#if QT_VERSION > 0x040600
#include <QPropertyAnimation>
#endif
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "backgroudimage.h"
#include "settings.h"
#include "saverestore.h"
#include "slideshow.h"
#include "fromfs.h"
#include "defines.h"
#include "onlinecheck.h"

#include <QtGlobal>

#include "ui_ffmpegfrontend.h"

QWidget *MainWindow::pointerToMainWindow=0;
//imageWidget *MainWindow::viewer=0;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    copy_application_files();

    settings conf;

    ui->setupUi(this);

    // For now firefox monitor works only in linux
#if defined(Q_WS_WIN) || defined(Q_WS_PM)
    ui->PB_monitor->hide();
#endif

    translator = new QTranslator();
    onCB_language_changed(conf.conf->value("GoogleSearch/language",0).toInt());


    //Set TE_Message with 3 EOL for managing messages
    ui->TE_messages->setText("\n\n\n");
    pointerToMainWindow=this;

    //    setAttribute(Qt::WA_DeleteOnClose);
    connect(ui->LE_searchString, SIGNAL(returnPressed()),
            this, SLOT(on_PB_search_clicked()));

    _about=new about(this->ui->centralWidget);

    browser=new browse(this);
    browser->hide();
    ui->verticalLayout->insertWidget(1,browser);
    connect(browser, SIGNAL(button_close_pressed()), this, SLOT(onBrowserButtonClosePressed()));

    // Setup mainwindow ffmpeg parameters
    ffmpeg=new ffmpegFrontend("",this);
    for (int i=0; i<ffmpeg->ui->CB_ffmpeg_preset->count(); i++ )
        ui->CB_ffmpeg_preset->addItem(ffmpeg->ui->CB_ffmpeg_preset->itemText(i));
    ui->CB_ffmpeg_preset->setCurrentIndex(ffmpeg->ui->CB_ffmpeg_preset->currentIndex());
    ui->LE_ffmpe_params->setText(ffmpeg->ui->TE_preview_args->toPlainText());
    connect(ui->CB_ffmpeg_preset, SIGNAL(currentIndexChanged(int)), this, SLOT(onCB_ffmpeg_preset_changed(int)) );
    connect(ffmpeg, SIGNAL(preset_or_args_changed()), this, SLOT(onFfmpeg_args_changed()) );

    down=new save(ffmpeg);
    connect(down, SIGNAL(downloadsComplete()), this, SLOT(onEnableDownloadButtonRequest()) );
    connect(down, SIGNAL(downloadsComplete()), this, SLOT(onEnableSearchButtonRequest()) );

    // Restore previous settings
    this->restoreGeometry(conf.conf->value("main/geometry").toByteArray());

    // Setup GraphicsView
    ui->LE_zoom->show();
    ui->PB_addRow->show();
    ui->PB_removeRow->show();
    viewer = new imageWidget();
    ui->verticalLayout->insertWidget(1,viewer);

    connect(viewer->scene, SIGNAL(item_browse_button_clicked(const QString &)) ,
            this, SLOT(onBrowseItem(const QString &)) );
    connect(viewer->scene, SIGNAL(item_download_button_clicked(imageContainer*)),
            this, SLOT(onDownloadItem(imageContainer*)) );
    connect(viewer->scene, SIGNAL(item_setAsBackground_button_clicked(imageContainer*)),
            this, SLOT(onSetAsBackgroundItem(imageContainer*)) );
    connect(viewer->scene, SIGNAL(item_playWithMplayer_button_clicked(imageContainer*)),
            this, SLOT(onPlayWithMplayerItem(imageContainer*)) );
    connect(viewer->scene, SIGNAL(networking_finished()),
            this, SLOT(onEnableSearchButtonRequest()) );
    connect(viewer->scene, SIGNAL(outMessage(const QString &,int)),
            this, SLOT(onOutMessage(const QString &,int)) );
    connect(viewer->scene, SIGNAL(onScrollRightLimitReached()),
            this, SLOT(onScrollRightLimitReached()) );
    connect(viewer, SIGNAL(PB_open_download_directory_clicked()),
            this, SLOT(onPB_open_download_directory_clicked()) );

    // TODO: on OS X seems that openGL stay over the mediaplayer

    ui->LE_searchString->setText(                   conf.conf->value("main/searchString","").toString() );
    ui->CB_and_clear->setChecked(                   conf.conf->value("main/searchAndClear",true).toBool() );
    ui->CB_and_download->setChecked(                conf.conf->value("main/searchAndDownload",false).toBool() );
    ui->CB_autoAdd->setChecked(                     conf.conf->value("main/autoAdd",true).toBool() );
    ui->LE_curr_dir->setText(                       conf.conf->value("main/downloadDir",QDir::currentPath()).toString() );
    ui->LE_dir_to_import->setText(                  conf.conf->value("main/importDir",QDir::currentPath()).toString() );
    ui->CB_include_subdir->setChecked(              conf.conf->value("main/includeSubdir",false).toBool());
    ui->RB_use_numbers->setChecked(                 conf.conf->value("main/fileNaming1",true).toBool());
    ui->RB_use_numbers_and_original_name->setChecked(conf.conf->value("main/fileNaming2",false).toBool());
    ui->RB_use_original_name->setChecked(           conf.conf->value("main/fileNaming3",false).toBool());
    ui->tabWidget->setCurrentIndex(                 conf.conf->value("main/tab",0).toInt());
    ui->SB_max_concurrent_downloads->setValue(      conf.conf->value("main//maxDownloads",5).toInt());
    network::max_cocurrent_downlods=ui->SB_max_concurrent_downloads->value();

    ui->SB_images_to_get->setText(                  conf.conf->value("GoogleSearch/nResults","20").toString());
    ui->SB_start_image_number->setText(             conf.conf->value("GoogleSearch/startFrom","0").toString());
    ui->RB_method_ajax->setChecked(                 conf.conf->value("GoogleSearch/methodAjax",false).toBool());
    ui->RB_method_public->setChecked(               conf.conf->value("GoogleSearch/methodPublic",true).toBool());
    ui->CB_content_type->setCurrentIndex(           conf.conf->value("GoogleSearch/contentType",0).toInt());
    ui->CB_size->setCurrentIndex(                   conf.conf->value("GoogleSearch/contentSize",0).toInt());
    ui->CB_coloration->setCurrentIndex(             conf.conf->value("GoogleSearch/contentColoration",0).toInt());
    ui->CB_domain->setItemText(                0   ,conf.conf->value("GoogleSearch/contentDomain","").toString());
    ui->CB_filter->setCurrentIndex(                 conf.conf->value("GoogleSearch/contentFilter",1).toInt());
    ui->CB_license->setCurrentIndex(                conf.conf->value("GoogleSearch/contentLicense",0).toInt());
    ui->CB_language->setCurrentIndex(               conf.conf->value("GoogleSearch/language",0).toInt());

    ui->SB_feeds_to_get->setText(                   conf.conf->value("YouTubeSearch/nResults","20").toString());
    ui->SB_feed_start_number->setText(              conf.conf->value("YouTubeSearch/StartFrom","1").toString());
    // Force 1 if 0
    if (ui->SB_feed_start_number->text()=="0") ui->SB_feed_start_number->setText("1");
    ui->CB_feed_order->setCurrentIndex(             conf.conf->value("YouTubeSearch/feedOrder",0).toInt());
    ui->CB_feed_language->setCurrentIndex(          conf.conf->value("YouTubeSearch/feedLanguage",0).toInt());
    ui->CB_feed_time->setCurrentIndex(              conf.conf->value("YouTubeSearch/feedTime",0).toInt());
    ui->RB_only_mp4->setChecked(                    conf.conf->value("YouTubeSearch/onlyMp4",true).toBool());
    youTube::only_mp4=conf.conf->value("YouTubeSearch/onlyMp4",true).toBool();

    ui->LE_delay->setText(         conf.conf->value("Slideshow/delay",0).toString());
    ui->CB_fullscreen->setChecked( conf.conf->value("Slideshow/fullscreen",false).toBool());
    ui->CB_loop->setChecked(       conf.conf->value("Slideshow/loop",false).toBool());
    ui->RB_all->setChecked(        conf.conf->value("Slideshow/all",true).toBool());
    ui->RB_images_only->setChecked(conf.conf->value("Slideshow/imagesOnly",false).toBool());
    ui->RB_videos_only->setChecked(conf.conf->value("Slideshow/videosOnly",false).toBool());

    ui->CB_use_ffmpeg->setChecked(                   conf.conf->value("ffmpeg/convertWithFfmpeg",false).toBool());
    ui->CB_delete_original_file->setChecked(         conf.conf->value("ffmpeg/deleteOriginalFile",false).toBool());
    ffmpeg->ui->CB_as_original->setChecked(          conf.conf->value("ffmpeg/asOriginal",true).toBool());
    ffmpeg->ui->SB_x_resolution->setValue(           conf.conf->value("ffmpeg/width",640).toInt());
    ffmpeg->ui->SB_y_resolution->setValue(           conf.conf->value("ffmpeg/height",480).toInt());
    ffmpeg->ui->SB_threads->setValue(                conf.conf->value("ffmpeg/threads",-1).toInt());
    ffmpeg->ui->SB_x_resolution->setDisabled(ffmpeg->ui->CB_as_original->isChecked());
    ffmpeg->ui->SB_y_resolution->setDisabled(ffmpeg->ui->CB_as_original->isChecked());
    on_CB_delete_original_file_clicked();

    ui->LE_saverestore_dir->setText( conf.conf->value("Saverestore/saverestoreFile","").toString() );

    connect(ui->CB_language, SIGNAL(currentIndexChanged(int)), this, SLOT(onCB_language_changed(int)) );

    // Set windows title
    this->setWindowTitle("Media Downloader - v"+QString(VERSION_NUMBER)+" "+QString(VERSION_REVISION));

    // Check phonon capabilities
#ifdef PHONON_LIB
    mediaPlayer::checkPhononCapabilities();
    if (!mediaPlayer::phonon_capabilities.flv || !mediaPlayer::phonon_capabilities.mp4)
    {
        QString s=tr("Your system is not able to play");
        s+= (!mediaPlayer::phonon_capabilities.flv ? " flv\n" : "");
        s+= (!mediaPlayer::phonon_capabilities.mp4 ? " mp4\n" : "");
        s+= tr("video codec must be installed");
        QMessageBox::warning(this, "warning", s, QMessageBox::Ok);
    }
#endif

    // Fill preferred video resolution combo box
    for (int i=0; i<9; i++) ui->CB_max_resolution->addItem(QString::number(youTube::videoFormats[i].fmt)+"-"+
                                                           youTube::videoFormats[i].res+"-"+
                                                           youTube::videoFormats[i].container);
    ui->CB_max_resolution->setCurrentIndex( conf.conf->value("YouTubeSearch/maxResolution",0).toInt() );
    youTube::ID_preferred_video_format=conf.conf->value("YouTubeSearch/maxResolution",0).toInt();
    connect( ui->CB_max_resolution, SIGNAL(currentIndexChanged(int)), this, SLOT(onCB_max_resolution_changed(int)) );

    // Set connection for RB_videos_only
    connect( ui->RB_only_mp4, SIGNAL(clicked(bool)), this, SLOT(on_RB_only_mp4_clicked(bool)) );

    // Set validator for slideshow delay
    validator = new QIntValidator(0,999,ui->LE_delay);
    ui->LE_delay->setValidator(validator);
    connect(ui->LE_delay, SIGNAL(returnPressed()), this, SLOT(on_PB_slide_start_clicked()));

    // Set this validator also for other input lineEdit
    validator2 = new QIntValidator(1,999,this);
    ui->SB_feed_start_number->setValidator(validator2);
    connect(ui->SB_feed_start_number, SIGNAL(textChanged(QString)), this, SLOT(onSB_feed_start_number_text_changed(QString)));
    ui->SB_feeds_to_get->setValidator(validator);
    ui->SB_images_to_get->setValidator(validator);
    ui->SB_start_image_number->setValidator(validator);

    // Set content of styles combo box
    fill_CB_style();

    isMonitoringFirefoxCache = false;
    addByScrolling = false;
    start_image_number  = ui->SB_start_image_number->text().toInt();
    images_to_get       = ui->SB_images_to_get->text().toInt();
    feed_start_number   = ui->SB_feed_start_number->text().toInt();
    feeds_to_get        = ui->SB_feeds_to_get->text().toInt();

    // enabling google suggestions
    ui->LE_searchString->setCanSuggest(true);
}

void MainWindow::fill_CB_style()
{
    settings conf;

    ui->CB_style->clear();
    ui->CB_style->addItem(tr("System style"));
    foreach (QFileInfo i, styles::getCssFileList()) {
        ui->CB_style->addItem(i.baseName());
        ui->CB_style->addItem(i.baseName()+tr("+watch"));
    }
    QString s = conf.conf->value("main/style","").toString();
    int i = ui->CB_style->findText(s);
    style=new styles(ui->CB_style->itemText(i).toAscii(), this);
    connect(ui->CB_style,    SIGNAL(currentIndexChanged(int)), this, SLOT(onCB_style_changed(int)) );
    ui->CB_style->setCurrentIndex( (i<0 ? 0 : i) );
}

MainWindow::~MainWindow()
{
    settings conf;

    conf.store_settings("main/geometry", this->saveGeometry());
    conf.store_settings("main/tab", ui->tabWidget->currentIndex());
    conf.store_settings("main/style", ui->CB_style->currentText());
    conf.store_settings("main/searchString", ui->LE_searchString->text() );
    conf.store_settings("main/searchAndClear", ui->CB_and_clear->isChecked() );
    conf.store_settings("main/searchAndDownload", ui->CB_and_download->isChecked() );
    conf.store_settings("main/autoAdd", ui->CB_autoAdd->isChecked() );
    conf.store_settings("main/downloadDir", ui->LE_curr_dir->text());
    conf.store_settings("main/importDir", ui->LE_dir_to_import->text());
    conf.store_settings("main/includeSubdir", ui->CB_include_subdir->isChecked());
    conf.store_settings("main/fileNaming1", ui->RB_use_numbers->isChecked());
    conf.store_settings("main/fileNaming2", ui->RB_use_numbers_and_original_name->isChecked());
    conf.store_settings("main/fileNaming3", ui->RB_use_original_name->isChecked());
    conf.store_settings("main//maxDownloads", ui->SB_max_concurrent_downloads->value());

    conf.store_settings("GoogleSearch/nResults", ui->SB_images_to_get->text());
    conf.store_settings("GoogleSearch/startFrom", ui->SB_start_image_number->text());
    conf.store_settings("GoogleSearch/methodAjax", ui->RB_method_ajax->isChecked());
    conf.store_settings("GoogleSearch/methodPublic", ui->RB_method_public->isChecked());
    conf.store_settings("GoogleSearch/language", ui->CB_language->currentIndex());
    conf.store_settings("GoogleSearch/contentType", ui->CB_content_type->currentIndex());
    conf.store_settings("GoogleSearch/contentSize", ui->CB_size->currentIndex());
    conf.store_settings("GoogleSearch/contentColoration", ui->CB_coloration->currentIndex());
    conf.store_settings("GoogleSearch/contentDomain", ui->CB_domain->currentText());
    conf.store_settings("GoogleSearch/contentFilter", ui->CB_filter->currentIndex());
    conf.store_settings("GoogleSearch/contentLicense", ui->CB_license->currentIndex());

    conf.store_settings("YouTubeSearch/nResults", ui->SB_feeds_to_get->text());
    conf.store_settings("YouTubeSearch/StartFrom", ui->SB_feed_start_number->text());
    conf.store_settings("YouTubeSearch/feedOrder", ui->CB_feed_order->currentIndex());
    conf.store_settings("YouTubeSearch/feedLanguage", ui->CB_feed_language->currentIndex());
    conf.store_settings("YouTubeSearch/feedTime", ui->CB_feed_time->currentIndex());
    conf.store_settings("YouTubeSearch/maxResolution", ui->CB_max_resolution->currentIndex());
    conf.store_settings("YouTubeSearch/onlyMp4", ui->RB_only_mp4->isChecked());

    conf.store_settings("Slideshow/delay",      ui->LE_delay->text().toInt());
    conf.store_settings("Slideshow/fullscreen", ui->CB_fullscreen->isChecked());
    conf.store_settings("Slideshow/loop",       ui->CB_loop->isChecked());
    conf.store_settings("Slideshow/all",        ui->RB_all->isChecked());
    conf.store_settings("Slideshow/imagesOnly", ui->RB_images_only->isChecked());
    conf.store_settings("Slideshow/videosOnly", ui->RB_videos_only->isChecked());

    conf.store_settings("Saverestore/saverestoreFile", ui->LE_saverestore_dir->text());

    conf.store_settings("ffmpeg/convertWithFfmpeg"  ,ui->CB_use_ffmpeg->isChecked());
    conf.store_settings("ffmpeg/deleteOriginalFile" ,ui->CB_delete_original_file->isChecked());
    conf.store_settings("ffmpeg/asOriginal"         ,ffmpeg->ui->CB_as_original->isChecked());
    conf.store_settings("ffmpeg/width"              ,ffmpeg->ui->SB_x_resolution->value());
    conf.store_settings("ffmpeg/height"             ,ffmpeg->ui->SB_y_resolution->value());
    conf.store_settings("ffmpeg/threads"            ,ffmpeg->ui->SB_threads->value());

    delete validator;
    delete validator2;
    delete style;
    delete translator;
    delete down;
    delete ffmpeg;
    delete _about;
    delete browser;
    delete viewer;
    delete ui;
}


void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    int i[8];
    QString s;
    switch (e->type()) {
    case QEvent::LanguageChange:
        // Save and restore state of comboBoxes because after translation the currIndex is reset to 0
        ui->CB_language->disconnect();

        s=ui->CB_domain->currentText();
        i[0] = ui->CB_content_type->currentIndex();
        i[1] = ui->CB_size->currentIndex();
        i[2] = ui->CB_coloration->currentIndex();
        i[3] = ui->CB_filter->currentIndex();
        i[4] = ui->CB_license->currentIndex();
        i[5] = ui->CB_language->currentIndex();
        i[6] = ui->CB_feed_order->currentIndex();
        i[7] = ui->CB_feed_time->currentIndex();
        ui->retranslateUi(this);
        fill_CB_style();
        ui->CB_domain->setItemText(0,s);
        ui->CB_content_type->setCurrentIndex(i[0]);
        ui->CB_size->setCurrentIndex(        i[1]);
        ui->CB_coloration->setCurrentIndex(  i[2]);
        ui->CB_filter->setCurrentIndex(      i[3]);
        ui->CB_license->setCurrentIndex(     i[4]);
        ui->CB_language->setCurrentIndex(    i[5]);
        ui->CB_feed_order->setCurrentIndex(  i[6]);
        ui->CB_feed_time->setCurrentIndex(   i[7]);

        connect(ui->CB_language, SIGNAL(currentIndexChanged(int)), this, SLOT(onCB_language_changed(int)));
        break;
    default:
        break;
    }
}

void MainWindow::onCB_language_changed(int i)
{
    switch (i) {
    case 0: translator->load(""); break;
    case 1: translator->load(":/translations/translations/md_it"); break;
    case 2: translator->load(":/translations/translations/md_ru"); break;
    case 3: translator->load(":/translations/translations/md_ua"); break;
    case 4: translator->load(":/translations/translations/md_de"); break;
    default: return;
    }

    QApplication::installTranslator(translator);
}

void MainWindow::onPB_open_download_directory_clicked()
{
    QDesktopServices::openUrl( QUrl(ui->LE_curr_dir->text()) );
}

// add items if while scrolling the view we reached the right edge
void MainWindow::onScrollRightLimitReached()
{
    if (!ui->CB_autoAdd->isChecked()) viewer->scene->stopInertia();
    if (addByScrolling || network::n_current_downloads>0) return;


    if (ui->CB_autoAdd->isChecked() &&
            viewer->scene->n_widget>0 &&
            !ui->LE_searchString->text().isEmpty() &&
            (ui->tabWidget->currentWidget()->objectName()=="tab_google_image" ||
             ui->tabWidget->currentWidget()->objectName()=="tab_YouTube")
            ) {
        viewer->scene->expandWidth();
        addByScrolling = true;

        // reimpost scenerect if no items are found
        if (doSearch(QStringList(ui->LE_searchString->text()))==0) {
            viewer->scene->sceneOGL.setSceneRect(viewer->scene->sceneOGL.itemsBoundingRect());
            viewer->scene->stopInertia();
        }
    }

    addByScrolling = false;
}

void MainWindow::copy_application_files()
{
    // If we are in release mode in lunux, we need to have .css and .ffpreset files copied into
    // ~/.meiadownloader/
    // try first to copy those files from /etc/mediadownloader , if this directory doesn'exists
    // try to copy from .pro file directory
    QDir path;
    QDir systemPath;
    QFile file;
    QFileInfoList entries;

#ifdef DEBUGGING
    // nothing to do, css and ffpreset are in .pro file directory
//    path.setPath(QString(QUOTEME(PWD_PRO))+"/css/");
#else
    #ifdef Q_WS_X11
        path.setPath(QDir::homePath()+"/.mediadownloader/");
    #else
        path.setPath(QDir::currentPath()+"/css/");
    #endif
    if (!path.exists()) path.mkdir(path.absolutePath());

    #ifdef Q_WS_X11
        path.setPath(QDir::homePath()+"/.mediadownloader/css/");
    #else
        path.setPath(QDir::currentPath()+"/css/");
    #endif
    if (!path.exists()) path.mkdir(path.absolutePath());

    #ifdef Q_WS_X11
        path.setPath(QDir::homePath()+"/.mediadownloader/ffmpeg-presets/");
    #else
        path.setPath(QDir::currentPath()+"/ffmpeg-presets/");
    #endif
    if (!path.exists()) path.mkdir(path.absolutePath());


    #ifdef Q_WS_X11
        systemPath.setPath("/etc/mediadownloader/css/");
        entries= systemPath.entryInfoList(QStringList("*.css"),QDir::Files,QDir::Name);
        // The copy will not overwrite existing files
        foreach (QFileInfo f, entries) {
            file.setFileName(f.absoluteFilePath());
            file.copy(QDir::homePath()+"/.mediadownloader/css/"+f.fileName());
        }

        systemPath.setPath("/etc/mediadownloader/ffmpeg-presets/");
        entries= systemPath.entryInfoList(QStringList("*.ffpreset"),QDir::Files,QDir::Name);
        foreach (QFileInfo f, entries) {
            file.setFileName(f.absoluteFilePath());
            file.copy(QDir::homePath()+"/.mediadownloader/ffmpeg-presets/"+f.fileName());
        }
    #endif

    #ifdef Q_WS_WIN
        // Nothing to do, css will install in program directory
    #endif
#endif
}

void MainWindow::onCB_style_changed(int i)
{
    QString s=ui->CB_style->itemText(i).toAscii();
    style->setStyle(s,s.contains(tr("+watch")));
}

void MainWindow::onSB_feed_start_number_text_changed(QString s)
{
    if (s=="0") ui->SB_feed_start_number->setText("");
}

void MainWindow::on_RB_only_mp4_clicked(bool b)
{
    youTube::only_mp4=b;
}

void MainWindow::onCB_max_resolution_changed(int i)
{
    youTube::ID_preferred_video_format=i;

    // find the new preferred video resolution for those video items that are already set video_info
    for (int n=0; n<viewer->scene->itemsList.size(); n++)
    if (viewer->scene->itemsList.at(n)->image.youTubeInfo.video_info.size()!=0)
        viewer->scene->itemsList.at(n)->image.youTubeInfo.ID_preferred_video_format=
           youTube::getPreferredResolution(viewer->scene->itemsList.at(n)->image.youTubeInfo, i);
}

void MainWindow::onPlayWithMplayerItem(imageContainer *item)
{
    QStringList args;
    if (item->image.kind==CONTAINER_KIND_VIDEO) {
        if (item->image.youTubeInfo.video_info.size()==0)
            if (youTube::getTubeVideoInfo(item->image.youTubeInfo)==0) return;
        args << item->image.youTubeInfo.video_info.at(item->image.youTubeInfo.ID_preferred_video_format).url_play;
    }
    if (item->image.kind==CONTAINER_KIND_VIDEO_OTHER || item->image.kind==CONTAINER_KIND_VIDEO_LOCAL)
        args << item->image.image_url;
    QProcess::startDetached ( "mplayer", args );
}

void MainWindow::onSetAsBackgroundItem(imageContainer *item)
{
    backgroudImage b(item->image.image_url);
}

void MainWindow::onBrowseItem(const QString url)
{
    viewer->hide();
    browser->setUrl(url);
    browser->show();
}

void MainWindow::onDownloadItem(imageContainer *item)
{
    viewer->scene->disableContextMenuEvent();
    ui->PB_download->setDisabled(true);
    ui->PB_search->setDisabled(true);
    if (ui->RB_use_numbers->isChecked())
        down->download_one(item->image,
                          ui->LE_curr_dir->text(),
                          save::USE_NUMBER,
                          ui->SB_start_image_number->text().toInt(),
                          ui->CB_use_ffmpeg->isChecked());
    if (ui->RB_use_numbers_and_original_name->isChecked())
        down->download_one(item->image,
                          ui->LE_curr_dir->text(),
                          save::USE_NUMBER_PLUS_FILENAME,
                          ui->SB_start_image_number->text().toInt(),
                          ui->CB_use_ffmpeg->isChecked());
    if (ui->RB_use_original_name->isChecked())
        down->download_one(item->image,
                          ui->LE_curr_dir->text(),
                          save::USE_FILENAME,
                          ui->SB_start_image_number->text().toInt(),
                          ui->CB_use_ffmpeg->isChecked());
    ui->PB_download->setDisabled(false);
    viewer->scene->enableContextMenuEvent();
}

void MainWindow::on_PB_directory_clicked()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::Directory);
    dialog.setViewMode(QFileDialog::List);
    dialog.setDirectory(ui->LE_curr_dir->text());
    QString s=dialog.getExistingDirectory(this);
    if (!s.isEmpty()) ui->LE_curr_dir->setText(s);
}

// Convert GoogleImage image structure to be compatible with imagecontainer
// It need 3 rows of description and the 2 urls for contextual menu
QList<imageStruct> MainWindow::convert_GoogleStruct_to_imageStruct(QList<GoogleImageStruct> google)
{
    QList<imageStruct> ret;
    imageStruct t;
    int i=0;
    while (i<google.size())
    {
        t.kind = CONTAINER_KIND_IMAGE;
        t.googleImageInfo=google.at(i);
//        t.youTubeInfo=NULL;
        t.description_row1=google.at(i).kind+"  "+google.at(i).descr;
        t.description_row2=google.at(i).contentNoFormatting;
        t.description_row3=google.at(i).visibleUrl;
        t.image_tburl=google.at(i).tbUrl;
        t.image_url=google.at(i).url;
        t.image_source=google.at(i).originalContextUrl;
        ret.append(t);
        i++;
    }
    i=ret.size();
    return ret;
}
QList<imageStruct> MainWindow::convert_youTube_to_imageStruct(QList<YouTubeStruct> tubes)
{
    QList<imageStruct> ret;
    imageStruct t;
    int i=0;
    while (i<tubes.size())
    {
        t.kind = CONTAINER_KIND_VIDEO;
//        t.googleImageInfo=NULL;
        t.youTubeInfo=tubes.at(i);
        t.description_row1=tubes.at(i).title;
        t.description_row2=tubes.at(i).content;
        t.description_row3=tubes.at(i).duration+"\""+" - "+tubes.at(i).author_name;
        t.image_tburl=tubes.at(i).p[3].url;  //4th thumbnail is that one with better resolution
        t.image_url=tubes.at(i).id;
        t.image_source=tubes.at(i).mediaUrl;
        ret.append(t);
        i++;
    }
    i=ret.size();
    return ret;
}

void MainWindow::on_PB_search_clicked()
{
    addByScrolling      = false;
    start_image_number  = ui->SB_start_image_number->text().toInt();
    images_to_get       = ui->SB_images_to_get->text().toInt();
    feed_start_number   = ui->SB_feed_start_number->text().toInt();
    feeds_to_get        = ui->SB_feeds_to_get->text().toInt();
    doSearch(QStringList(ui->LE_searchString->text()));

}

int MainWindow::doSearch(QStringList searchString)
{
    int items_found;
    QList<YouTubeStruct> tubes;

    if (ui->tabWidget->currentWidget()->objectName()=="tab_google_image") // We are in Google Image Search tab
    {
        if (ui->CB_and_clear->isChecked() && !addByScrolling)
            on_PB_clear_clicked();

        if (addByScrolling) {
            start_image_number += images_to_get;
            images_to_get       = viewer->scene->get_rows()*4;  // add 5 columns of items
            images_to_get      += viewer->scene->get_rows()-(viewer->scene->n_widget % viewer->scene->get_rows());
        }

        ui->PB_search->setDisabled(true);
        if (ui->RB_method_ajax->isChecked())   google=new GooglePicture(GooglePicture::METHOD_AJAX);
        if (ui->RB_method_public->isChecked()) google=new GooglePicture(GooglePicture::METHOD_PUBLIC);
        connect(google, SIGNAL(outMessage(QString,int)),
                this, SLOT(onOutMessage(QString,int)) );

        items_found=0;
        for (int i=0; i<searchString.size(); i++)
        {
            items_found+=google->get_images_list(searchString.at(i),
                                                 start_image_number, images_to_get,
                                                 GooglePicture::content_type[ui->CB_content_type->currentIndex()],
                                                 GooglePicture::image_size[ui->CB_size->currentIndex()],
                                                 GooglePicture::coloration[ui->CB_coloration->currentIndex()],
                                                 GooglePicture::safe_filter[ui->CB_filter->currentIndex()],
                                                 GooglePicture::content_rights[ui->CB_license->currentIndex()],
                                                 ui->CB_domain->currentText()
                                                 );
            onOutMessage(tr("Images searched ")+QString::number(items_found)+
                         tr(" of about ")+QString::number(google->estimatedResultCount),1);
        }
        if (items_found>0)
        {
            if (ui->CB_and_download->isChecked() && !addByScrolling)
                downloadItemsList( convert_GoogleStruct_to_imageStruct(google->images) );
            else
                viewer->scene->add_all_images( convert_GoogleStruct_to_imageStruct(google->images) );
        }
        else
            viewer->scene->sceneOGL.setSceneRect(viewer->scene->sceneOGL.itemsBoundingRect());
        delete google;
    }

    if (ui->tabWidget->currentWidget()->objectName()=="tab_YouTube") // We are in YouTube search tab
    {
        if (ui->CB_and_clear->isChecked() && !addByScrolling)
            on_PB_clear_clicked();

        if (addByScrolling) {
            feed_start_number += feeds_to_get;
            feeds_to_get       = viewer->scene->get_rows()*4;  // add 5 columns of items
            feeds_to_get      += viewer->scene->get_rows()-(viewer->scene->n_widget % viewer->scene->get_rows());
        }

        ui->PB_search->setDisabled(true);
        tubes=youTube::getList(searchString.at(0),
                               VIDEO_FEED_NONE_USE_SEARCH,
                               feed_start_number,
                               feeds_to_get,
                               ui->CB_feed_order->currentIndex(),
                               ui->CB_feed_language->currentText().left(2),
                               ui->CB_feed_time->currentIndex()
                               );
        if (tubes.size()>0)
        {
            if (ui->CB_and_download->isChecked() && !addByScrolling)
                downloadItemsList( convert_youTube_to_imageStruct(tubes) );
            else
                viewer->scene->add_all_images( convert_youTube_to_imageStruct(tubes) );
        }
        onOutMessage(tr("Video searched ")+QString::number(tubes.size()),1);
    }

    ui->PB_search->setDisabled(false);
    return items_found;
}

void MainWindow::onEnableSearchButtonRequest()
{
    ui->PB_search->setDisabled(false);
}

void MainWindow::onEnableDownloadButtonRequest()
{
    ui->PB_download->setDisabled(false);
}

void MainWindow::on_PB_exit_clicked()
{
    this->close();
}

void MainWindow::on_PB_clear_clicked()
{
    viewer->scene->clear_scene();
    onOutMessage("",0);
}


void MainWindow::on_PB_addRow_clicked()
{
    viewer->scene->add_row();
}

void MainWindow::on_PB_removeRow_clicked()
{
    viewer->scene->remove_row();
}

void MainWindow::on_PB_about_clicked()
{
    QRect start, end;

    if (!_about->isHidden()) {_about->hide(); return; }
    end.setRect(this->x()+(this->width() - _about->width())/2 ,
                this->y()+(this->height() - _about->height())/2 ,
                _about->width(), _about->height() );

    start.setRect(this->x()+this->width()/2, this->y()+this->height()/2, 0,0);
#if QT_VERSION > 0x040600
    QPropertyAnimation *zoom = new QPropertyAnimation(_about, "geometry");
    zoom->setDuration(500);
    zoom->setStartValue(start);
    zoom->setEndValue(end);
    zoom->setEasingCurve(QEasingCurve(QEasingCurve::OutCubic));
    zoom->start();
#endif
    _about->show();
}

void MainWindow::onBrowserButtonClosePressed()
{
    browser->stopLoading();
    browser->close();
    viewer->show();
}

void MainWindow::onOutMessage(const QString &message, int row)
{
    if (row>2 || row<0) return; // max 3 rows
    static QString m[3]={"","",""};

    m[row]=message;
    ui->TE_messages->clear();
    ui->TE_messages->append(m[0]);
    ui->TE_messages->append(m[1]);
    ui->TE_messages->append(m[2]);
    QByteArray s;
    s = ui->TE_messages->toPlainText().toAscii();
    int a;
    a=2;
}


void MainWindow::on_PB_hide_clicked()
{
    if (ui->frame->isHidden()) {
        ui->PB_hide->setIcon(QIcon(":/icons/icons/arrow_up.png"));
        ui->PB_hide->setIconSize(QSize(16,48));
        ui->frame->show();
    }
    else {
        ui->PB_hide->setIcon(QIcon(":/icons/icons/arrow_down.png"));
        ui->PB_hide->setIconSize(QSize(16,48));
        ui->frame->hide();
    }
}

/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////
// YouTube tab
void MainWindow::getVideoFeed(StandardVideoFeeds feed)
{
    QList<YouTubeStruct> tubes;
    ui->PB_search->setDisabled(true);
    tubes=youTube::getList("",
                          feed,
                          ui->SB_feed_start_number->text().toInt(),
                          ui->SB_feeds_to_get->text().toInt(),
                          ui->CB_feed_order->currentIndex(),
                          ui->CB_feed_language->currentText().left(2),
                          ui->CB_feed_time->currentIndex());
    if (tubes.size()>0)
        viewer->scene->add_all_images( convert_youTube_to_imageStruct(tubes) );

    onOutMessage(tr("Video searched ")+QString::number(tubes.size()),0);
    ui->PB_search->setDisabled(false);
}

void MainWindow::on_PB_topRated_clicked()
{
    this->getVideoFeed(VIDEO_FEED_TOP_RATED);
}

void MainWindow::on_PB_topFavorites_clicked()
{
    this->getVideoFeed(VIDEO_FEED_TOP_FAVORITES);
}

void MainWindow::on_PB_mostViewed_clicked()
{
    this->getVideoFeed(VIDEO_FEED_MOST_VIEWED);
}

void MainWindow::on_PB_mostPopular_clicked()
{
    this->getVideoFeed(VIDEO_FEED_MOST_POPULAR);
}

void MainWindow::on_PB_mostRecent_clicked()
{
    this->getVideoFeed(VIDEO_FEED_MOST_RECENT);
}

/////////////////////////////////////////
/////////////////////////////////////////
// Save Restore tab
void MainWindow::on_PB_save_to_new_file_clicked()
{
    saverestore *s= new saverestore(saverestore::SAVE_TO_NEW_FILE, viewer->scene, ui->LE_saverestore_dir->text());
    delete s;
}

void MainWindow::on_PB_replace_all_to_saved_clicked()
{
    saverestore *s= new saverestore(saverestore::RESTORE_AND_REPLACE, viewer->scene, ui->LE_saverestore_dir->text());
    delete s;
}

void MainWindow::on_PB_append_to_existing_file_clicked()
{
    saverestore *s= new saverestore(saverestore::SAVE_AND_APPEND, viewer->scene, ui->LE_saverestore_dir->text());
    delete s;
}

void MainWindow::on_PB_append_saved_to_current_clicked()
{
    saverestore *s= new saverestore(saverestore::RESTORE_AND_APPEND, viewer->scene, ui->LE_saverestore_dir->text());
    delete s;
}

void MainWindow::on_PB_saverestore_clicked()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::AnyFile);
    dialog.setNameFilter(tr("Media Downloader file (*.md)"));
    dialog.setViewMode(QFileDialog::Detail);
    dialog.setLabelText(QFileDialog::FileName, tr("Choose file to be managed or write the file name"));


    if (dialog.exec())
    {
        saverestore_fileName.setFile(dialog.selectedFiles().at(0));
        if (saverestore_fileName.suffix()!="md") saverestore_fileName.setFile(saverestore_fileName.absoluteFilePath()+".md");
        if (saverestore_fileName.baseName()!="")
            ui->LE_saverestore_dir->setText(saverestore_fileName.absoluteFilePath());
        else
            ui->LE_saverestore_dir->setText("");
    }
}

/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////
// Slide show tab
void MainWindow::on_PB_slide_start_clicked()
{
    if (viewer->scene->itemsList.size()==0) return;
    new slideshow(viewer->scene->itemsList,
                                ui->LE_delay->text().toInt(),
                                ui->CB_fullscreen->isChecked(),
                                ui->CB_loop->isChecked(),
                                ui->RB_all->isChecked(),
                                ui->RB_images_only->isChecked(),
                                ui->RB_videos_only->isChecked(),
                                this);
}

/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////
// Import tab
void MainWindow::on_PB_start_import_clicked()
{
    fromfs fs(ui->CB_include_subdir->isChecked(),
              viewer->scene,
              this);
    connect(&fs, SIGNAL(outMessage(QString,int)),
            this, SLOT(onOutMessage(QString,int)));
    fs.parseFiles(QDir(ui->LE_dir_to_import->text()));
}

void MainWindow::on_PB_directory_to_import_clicked()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::Directory);
    dialog.setViewMode(QFileDialog::List);
    dialog.setDirectory(ui->LE_dir_to_import->text());
    QString s=dialog.getExistingDirectory(this);
    if (!s.isEmpty()) ui->LE_dir_to_import->setText(s);
}

void MainWindow::on_PB_check_updates_clicked()
{
    // Check online updates
    onlineCheck *check;
    check = new onlineCheck(true, this->centralWidget());
}

/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////
// Download tab
void MainWindow::downloadItemsList(QList<imageStruct> list)
{
    ui->PB_download->setDisabled(true);
    ui->PB_search->setDisabled(true);
    if (ui->RB_use_numbers->isChecked())
        down->download_all(list,
                          ui->LE_curr_dir->text(),
                          save::USE_NUMBER,
                          ui->SB_start_image_number->text().toInt(),
                          ui->CB_use_ffmpeg->isChecked());
    if (ui->RB_use_numbers_and_original_name->isChecked())
        down->download_all(list,
                          ui->LE_curr_dir->text(),
                          save::USE_NUMBER_PLUS_FILENAME,
                          ui->SB_start_image_number->text().toInt(),
                          ui->CB_use_ffmpeg->isChecked());
    if (ui->RB_use_original_name->isChecked())
        down->download_all(list,
                          ui->LE_curr_dir->text(),
                          save::USE_FILENAME,
                          ui->SB_start_image_number->text().toInt(),
                          ui->CB_use_ffmpeg->isChecked());
}

void MainWindow::on_PB_download_clicked()
{
    downloadItemsList(viewer->scene->getImageStructList());
}

void MainWindow::on_PB_multiple_search_from_file_clicked()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::AnyFile);
    dialog.setNameFilter(tr("Choose file to import")+"(*.txt *.csv)");
    dialog.setViewMode(QFileDialog::Detail);
    dialog.setLabelText(QFileDialog::FileName, tr("The search strings must be separated by semicolon or End Of Line"));


    if (dialog.exec())
    {
        QFile file(dialog.selectedFiles().at(0));
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
            QMessageBox::critical(0,QObject::tr("error"),QObject::tr("The file could not be read") );
            return;
        }
        QString line;
        QStringList searches;
        // Turn off remove clear scene options till all searches is done
        bool clearSceneChecked;
        clearSceneChecked = ui->CB_and_clear->isChecked();
        ui->CB_and_clear->setChecked(false);

        searches.clear();
        while (!file.atEnd()) {
            line = file.readLine().replace('\n',"").replace('\r',"");
            searches.append( line.split( ";" ) );
        }
        doSearch(searches);

        ui->CB_and_clear->setChecked(clearSceneChecked);
        file.close();
    }
}

// Checks the presence of ffmpeg
void MainWindow::on_CB_use_ffmpeg_clicked(bool checked)
{
    if (!checked) return;
    // If the process cannot be started, -2 is returned.
    // If the process crashes, -1 is returned. Otherwise,
    // the process' exit code is returned.

    int i = QProcess::execute ( "ffmpeg");
    if (i==-2)
    {
        QMessageBox::critical(0,QObject::tr("error"),QObject::tr("ffmpeg could not be found") );
        ui->CB_use_ffmpeg->setChecked(false);
    }
    else {
        if (!ffmpeg->check_ffmpeg_filters_availability()) {
            QMessageBox::critical(0,QObject::tr("error"),QObject::tr("ffmpeg has no scale or pad filter available.\nCan't convert videos") );
            ui->CB_use_ffmpeg->setChecked(false);
        }
    }
}

void MainWindow::on_PB_make_preset_clicked()
{
    if (ffmpeg->isHidden()) ffmpeg->show();
    else ffmpeg->hide();
}

void MainWindow::onCB_ffmpeg_preset_changed(int i)
{
    ffmpeg->ui->CB_ffmpeg_preset->setCurrentIndex(i);
}

void MainWindow::onFfmpeg_args_changed()
{
    ui->CB_ffmpeg_preset->setCurrentIndex(ffmpeg->ui->CB_ffmpeg_preset->currentIndex());
    ui->LE_ffmpe_params->setText(ffmpeg->ui->TE_preview_args->toPlainText());
}

void MainWindow::on_CB_delete_original_file_clicked()
{
    if (ui->CB_delete_original_file->isChecked())
        ffmpeg->delete_original_video=true;
    else
        ffmpeg->delete_original_video=false;
}

void MainWindow::on_SB_max_concurrent_downloads_valueChanged(int i)
{
    network::max_cocurrent_downlods=i;
}

///////////////////////////////////////////////////////
// Monitor firefox cache
void MainWindow::on_PB_monitor_clicked()
{
    if (isMonitoringFirefoxCache)
    {
        delete cacheMonitor;
        ui->PB_monitor->setText(tr("Start monitoring firefox"));
        isMonitoringFirefoxCache = false;
    }
    else {
        bool canMonitor;
        cacheMonitor = new firefoxMonitor(viewer->scene, &canMonitor, this);
        if (canMonitor) {
            ui->PB_monitor->setText(tr("Stop"));
            isMonitoringFirefoxCache = true;
            connect(cacheMonitor, SIGNAL(outMessage(QString,int)),
                    this, SLOT(onOutMessage(QString,int)) );
        }
        else
            delete cacheMonitor;
    }
}

