/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifdef PHONON_LIB
#include <kde4/phonon/backendcapabilities.h>
#endif
#include <QDebug>
#include <QMouseEvent>
#include <QApplication>
#include <QDesktopWidget>
#include <QStyle>
#include <QIcon>
#include <QImage>
#include "mediaplayer.h"
#include "popuptext.h"
#include "mainwindow.h"

// delay for updating values with a timer, seems that without there are random crashes
// I like the sliders have tracking, so implemented this timer
#define DELAY_BUG 200

clickableLabel::clickableLabel(QWidget *parent)
{
    Q_UNUSED(parent)
    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
    this->activateWindow();
}

void clickableLabel::mousePressEvent ( QMouseEvent *e ) {
    if (e->button() & Qt::LeftButton)
        emit clicked();
}
void clickableLabel::mouseDoubleClickEvent ( QMouseEvent * e ) {
    if (e->button() & Qt::LeftButton)
        emit doubleClicked();
}

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#ifdef PHONON_LIB
adjustVideoParameters::adjustVideoParameters(Phonon::VideoWidget * vw, QRect r, QWidget *parent) :
        QWidget(parent)
{
    this->setWindowFlags(Qt::WindowStaysOnTopHint);
//    this->setFocusPolicy(Qt::StrongFocus);
//    this->setFocus();
//    this->activateWindow();

    videoWidget = vw;

    this->setGeometry(r.x(), r.y(), r.width(), r.height());

    layout = new QGridLayout(this);
    brightness = new QSlider(this);
    contrast   = new QSlider(this);
    hue        = new QSlider(this);
    saturation = new QSlider(this);

    brightnessLabel = new QLabel(tr("brightness "),this);
    brightnessLabel->setMaximumSize(100,23);
    brightnessLabel->setMinimumSize(100,23);
    brightness->setRange(-100,100);
    brightness->setOrientation(Qt::Horizontal);
    brightness->setMaximumSize(175,15);
    brightness->setMinimumSize(175,15);
    brightness->setValue(vw->brightness()*100);
    connect(brightness, SIGNAL(valueChanged(int)), this, SLOT(onBrightness_moved(int)));

    contrastLabel = new QLabel(tr("contrast "),this);
    contrastLabel->setMaximumSize(100,23);
    contrastLabel->setMinimumSize(100,23);
    contrast->setRange(-100,100);
    contrast->setMaximumSize(175,15);
    contrast->setMinimumSize(175,15);
    contrast->setOrientation(Qt::Horizontal);
    contrast->setValue(vw->contrast()*100);
    connect(contrast, SIGNAL(valueChanged(int)), this, SLOT(onContrast_moved(int)));

    hueLabel = new QLabel(tr("hue "),this);
    hueLabel->setMaximumSize(100,23);
    hueLabel->setMinimumSize(100,23);
    hue->setMaximumSize(175,15);
    hue->setMinimumSize(175,15);
    hue->setRange(-100,100);
    hue->setOrientation(Qt::Horizontal);
    hue->setValue(vw->hue()*100);
    connect(hue, SIGNAL(valueChanged(int)), this, SLOT(onHue_moved(int)));

    saturationLabel = new QLabel(tr("saturation "),this);
    saturationLabel->setMaximumSize(100,23);
    saturationLabel->setMinimumSize(100,23);
    saturation->setMaximumSize(175,15);
    saturation->setMinimumSize(175,15);
    saturation->setRange(-100,100);
    saturation->setOrientation(Qt::Horizontal);
    saturation->setValue(vw->saturation()*100);
    connect(saturation, SIGNAL(valueChanged(int)), this, SLOT(onSaturation_moved(int)));

    PB_reset = new QPushButton(this);
    PB_reset->setMaximumSize(40,23);
    PB_reset->setMinimumSize(40,23);
    PB_reset->setObjectName(QString::fromUtf8("PB_reset"));
    PB_reset->setText("reset");
    connect(PB_reset, SIGNAL(clicked()), this, SLOT(onPB_reset_clicked()) );

    layout->addWidget(brightnessLabel,0,0);
    layout->addWidget(brightness,0,1);
    layout->addWidget(contrastLabel,1,0);
    layout->addWidget(contrast,1,1);
    layout->addWidget(hueLabel,2,0);
    layout->addWidget(hue,2,1);
    layout->addWidget(saturationLabel,3,0);
    layout->addWidget(saturation,3,1);
    layout->addWidget(PB_reset,4,1);

    this->setStyleSheet("QLabel {color: rgb(255,255,0); background-color: rgb(0,0,0);} "
                        "QSlider { margin: 0px; background-color: "
                        "qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, "
                        "stop:0 rgba(205, 0, 0, 255), stop:0.5 rgba(0, 0, 0, 180), "
                        "stop:1 rgba(0, 193, 79, 255)); border-radius: 4px;}");
    canUpdate=true;
}

adjustVideoParameters::~adjustVideoParameters()
{
    delete brightness;
    delete contrast;
    delete hue;
    delete saturation;
    delete brightnessLabel;
    delete contrastLabel;
    delete hueLabel;
    delete saturationLabel;
    delete layout;
}

void adjustVideoParameters::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    painter.fillRect( event->rect(), Qt::black );
    painter.setPen(Qt::yellow);
    painter.drawRect( event->rect().x(), event->rect().y(),
                      event->rect().width()-1, event->rect().height()-1);
}

void adjustVideoParameters::onTimerFinished()
{
    canUpdate=true;
}

void adjustVideoParameters::onPB_reset_clicked()
{
    onBrightness_moved(0);
    onContrast_moved(0);
    onHue_moved(0);
    onSaturation_moved(0);
}

void adjustVideoParameters::onBrightness_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        brightness->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setBrightness(0);
        return;
    }
    videoWidget->setBrightness( (qreal(i)/100.0) );
}

void adjustVideoParameters::onContrast_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        contrast->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setContrast(0);
        return;
    }

    videoWidget->setContrast( (qreal(i)/100.0) );
}

void adjustVideoParameters::onHue_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        hue->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setHue(0);
        return;
    }
    videoWidget->setHue( (qreal(i)/100.0) );
}

void adjustVideoParameters::onSaturation_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        saturation->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setSaturation(0);
        return;
    }
    videoWidget->setSaturation( (qreal(i)/100.0) );
}
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
adjustAudio::adjustAudio(Phonon::AudioOutput *ao, QRect r, QWidget *parent) :
    QWidget(parent, Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint)
{
    volumeSlider = new Phonon::VolumeSlider(this);
    volumeSlider->setObjectName(QString::fromUtf8("volumeSlider"));
    this->setGeometry(r);
    this->setMinimumSize(QSize(23, 265));
    this->setMaximumSize(QSize(23, 265));
    volumeSlider->setMinimumSize(QSize(23, 260));
    volumeSlider->setMaximumSize(QSize(23, 260));
    volumeSlider->setOrientation(Qt::Vertical);
    volumeSlider->setAudioOutput(ao);
    this->show();
}

adjustAudio::~adjustAudio()
{
    delete volumeSlider;
}

void adjustAudio::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    painter.fillRect( event->rect(), Qt::black );
    painter.setPen(Qt::yellow);
    painter.drawRect( event->rect().x(), event->rect().y(),
                      event->rect().width()-1, event->rect().height()-1);
}

#endif

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#ifdef PHONON_LIB
phononCapabilities mediaPlayer::phonon_capabilities={false,false};
#endif

mediaPlayer::mediaPlayer(bool accept_double_click, imageContainerKind container_kind, QWidget *parent) :
    QWidget(parent)
{
    parentWidget=parent;
    acceptDoubleClick=accept_double_click;

    this->setWindowTitle(QString::fromUtf8("player"));
    horizontalLayout_2 = new QHBoxLayout(this);
    horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
    horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));

    kind=container_kind;
    this->setupUi();

    setAttribute(Qt::WA_DeleteOnClose);

#if QT_VERSION > 0x040600
    zoom = new QPropertyAnimation(this, "geometry");
    connect(zoom, SIGNAL(finished()), this, SLOT(resizingFinished()));
#endif

    this->resize(QSize(0,0));
    resizing=false;
    isFullscreen=false;
}


mediaPlayer::~mediaPlayer()
{
    if (!pixContainer.isNull()) delete pixContainer;
#ifdef PHONON_LIB
    if ( (kind & CONTAINER_KIND_VIDEO) && !videoPlayer.isNull()) {
        videoPlayer->stop();
        videoPlayer->mediaObject()->clear();
        delete PB_close;
        delete volumeSliderButton;
        if (volumeSlider!=0) delete volumeSlider;
        delete seekSlider;
        delete videoPlayer;
        delete PB_adjustVideo;
        if (adjustVideoWidget!=0) delete adjustVideoWidget;
        delete PB_fullscreen;
        delete L_time;
        if (!CB_formats.isNull()) delete CB_formats;
        if (!PB_info.isNull()) delete PB_info;
        delete horizontalLayout;
        delete verticalLayout;
    }
#endif
    delete horizontalLayout_2;
#if QT_VERSION > 0x040600
    delete zoom;
#endif
}

void mediaPlayer::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    painter.fillRect( event->rect(), Qt::black );
}
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
void mediaPlayer::setupUi()
{
    if (kind & CONTAINER_KIND_IMAGE)
    {
#ifdef PHONON_LIB
        if (!videoPlayer.isNull()) {
            videoPlayer->stop();
            videoPlayer->mediaObject()->clear();
            delete PB_close;
            delete seekSlider;
            delete volumeSliderButton;
            if (volumeSlider!=0) delete volumeSlider;
            delete PB_adjustVideo;
            if (adjustVideoWidget!=0) delete adjustVideoWidget;
            delete videoPlayer;
            delete PB_fullscreen;
            delete PB_playPause;
            if (!CB_formats.isNull()) delete CB_formats;
            if (!PB_info.isNull()) delete PB_info;
            delete L_time;
            delete horizontalLayout;
            delete verticalLayout;
        }
#endif
        pixContainer = new clickableLabel(this);
        if (acceptDoubleClick)
            connect(pixContainer, SIGNAL(doubleClicked()), this, SLOT(close()));
    //        connect(pixContainer, SIGNAL(keyPressed(QKeyEvent*)), this, SLOT(onKeyPressed(QKeyEvent*)) );
        horizontalLayout_2->addWidget(pixContainer);
    }
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO)
    {
        if (!pixContainer.isNull()) delete pixContainer;
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(2);
        horizontalLayout->setContentsMargins(2, 2, 2, 2);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetNoConstraint);
        verticalLayout->addLayout(horizontalLayout);
        horizontalLayout_2->addLayout(verticalLayout);

        if (kind==CONTAINER_KIND_VIDEO)
        {
            PB_info = new QPushButton(this);
            PB_info->setObjectName(QString::fromUtf8("PB_info"));
            PB_info->setMinimumSize(QSize(26, 26));
            PB_info->setMaximumSize(QSize(26, 26));
            PB_info->setIcon(QIcon(":/icons/icons/info_green.png"));
            PB_info->setIconSize(QSize(23, 23));
            horizontalLayout->addWidget(PB_info);
            connect(PB_info, SIGNAL(clicked()), this, SLOT(popupInfo()));
        }

        L_time = new QLabel(this);
        L_time->setObjectName(QString::fromUtf8("L_time"));
        L_time->setMinimumSize(QSize(75, 23));
        L_time->setText("");
        L_time->setFrameShape(QFrame::Panel);
        horizontalLayout->addWidget(L_time);

        if (kind==CONTAINER_KIND_VIDEO)
        {
            CB_formats = new QComboBox(this);
            CB_formats->setObjectName(QString::fromUtf8("CB_formats"));
            CB_formats->setMinimumSize(QSize(160, 30));
            CB_formats->setMaximumSize(QSize(160, 30));
            horizontalLayout->addWidget(CB_formats);
        }

        PB_fullscreen = new QPushButton(this);
        PB_fullscreen->setObjectName(QString::fromUtf8("PB_info"));
        PB_fullscreen->setMinimumSize(QSize(26, 26));
        PB_fullscreen->setMaximumSize(QSize(26, 26));
        PB_fullscreen->setIcon(QIcon(":/icons/icons/fullscreen.png"));
        PB_fullscreen->setIconSize(QSize(23, 23));
        horizontalLayout->addWidget(PB_fullscreen);
        connect(PB_fullscreen, SIGNAL(clicked()), this, SLOT(switchFullScreen()));

        videoPlayer = new Phonon::VideoPlayer(Phonon::VideoCategory,this);
        videoPlayer->setObjectName(QString::fromUtf8("videoPlayer"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(10);
        sizePolicy.setHeightForWidth(videoPlayer->sizePolicy().hasHeightForWidth());
        videoPlayer->setSizePolicy(sizePolicy);
        videoPlayer->setMinimumSize(QSize(0, 75));
        videoPlayer->setStyleSheet("background: rgb(0,0,0);");
        verticalLayout->addWidget(videoPlayer);
        mediaObject=videoPlayer->mediaObject();
        connect(videoPlayer, SIGNAL(finished()), this, SLOT(onVideoPlayFinished()) );
        connect(mediaObject, SIGNAL(tick(qint64)), this, SLOT(tick(qint64)));
        connect(mediaObject, SIGNAL(bufferStatus(int)), this, SLOT(bufferChanged(int)) );

        PB_playPause = new QPushButton(this);
        PB_playPause->setObjectName(QString::fromUtf8("PB_info"));
        PB_playPause->setMinimumSize(QSize(26, 26));
        PB_playPause->setMaximumSize(QSize(26, 26));
        PB_playPause->setIcon(PB_playPause->style()->standardIcon(QStyle::SP_MediaPause));
        horizontalLayout->addWidget(PB_playPause);
        connect(PB_playPause, SIGNAL(clicked()), this, SLOT(switchPlayPause()));

        seekSlider = new Phonon::SeekSlider(this);
        seekSlider->setObjectName(QString::fromUtf8("seekSlider"));
        seekSlider->setMinimumSize(QSize(170, 0));
        seekSlider->setMaximumSize(QSize(16777215, 23));
        seekSlider->setIconVisible(false);
        horizontalLayout->addWidget(seekSlider);

        PB_adjustVideo = new QPushButton(this);
        PB_adjustVideo->setIcon(QIcon(":/icons/icons/graph.png"));
        connect(PB_adjustVideo, SIGNAL(clicked()), this, SLOT(onAdjustVideo_clicked()));
        horizontalLayout->addWidget(PB_adjustVideo);
        adjustVideoWidget=0;

        volumeSliderButton = new QPushButton(this);
        volumeSliderButton->setIcon(volumeSliderButton->style()->standardIcon(QStyle::SP_MediaVolume));
        connect(volumeSliderButton, SIGNAL(clicked()), this, SLOT(onVolumeSliderButton_clicked()));
        horizontalLayout->addWidget(volumeSliderButton);
        volumeSlider=0;

        PB_close = new QPushButton(this);
        PB_close->setObjectName(QString::fromUtf8("PB_close"));
        PB_close->setMinimumSize(QSize(50, 23));
        PB_close->setText(tr("close"));
        horizontalLayout->addWidget(PB_close);
        connect(PB_close, SIGNAL(clicked()), this, SLOT(close()));

        this->setStyleSheet("QLabel {color: rgb(255,255,0); background-color: rgb(0,0,0);} "
                            "QSlider { margin: 0px; background-color: rgb(120,120,120);}");

    }
#endif
    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
    this->activateWindow();
}

#ifdef PHONON_LIB
void mediaPlayer::onAdjustVideo_clicked()
{
    if (adjustVideoWidget!=0) {
        delete adjustVideoWidget;
        adjustVideoWidget=0;
    }
    else {
        adjustVideoWidget = new adjustVideoParameters(videoPlayer->videoWidget(),
                                                      QRect(PB_adjustVideo->x()-300,
                                                            PB_adjustVideo->y()+PB_adjustVideo->height(),
                                                            300,130),
                                                      this);
        adjustVideoWidget->show();
    }
}

void mediaPlayer::onVolumeSliderButton_clicked()
{
    if (volumeSlider!=0) {
        delete volumeSlider;
        volumeSlider=0;
    }
    else {
        volumeSlider = new adjustAudio(videoPlayer->audioOutput(),
                                                QRect(volumeSliderButton->geometry().x(),volumeSliderButton->geometry().y()+volumeSliderButton->height(),
                                                      volumeSliderButton->geometry().width(), volumeSliderButton->geometry().height() ),
                                                this);
//        volumeSliderContainer->setGeometry(volumeSliderButton->geometry().x(),volumeSliderButton->geometry().y()+volumeSliderButton->height(),
//                                  volumeSliderButton->geometry().width(), volumeSliderButton->geometry().height());
//        volumeSlider = new Phonon::VolumeSlider(volumeSliderContainer);
//        volumeSlider->setObjectName(QString::fromUtf8("volumeSlider"));
////        volumeSlider->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
////        volumeSlider->setGeometry(volumeSliderButton->geometry().x(),volumeSliderButton->geometry().y()+volumeSliderButton->height(),
////                                  volumeSliderButton->geometry().width(), volumeSliderButton->geometry().height());
//        volumeSliderContainer->setMinimumSize(QSize(23, 260));
//        volumeSliderContainer->setMaximumSize(QSize(23, 260));
//        volumeSlider->setOrientation(Qt::Vertical);
//        volumeSliderContainer->show();
//        volumeSlider->setAudioOutput(videoPlayer->audioOutput());
    }
}
#endif

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Handle KeyPress event for both mediaPlayer or pixContainer objects
void mediaPlayer::keyPressEvent( QKeyEvent * event )
{
    if (event->key() == Qt::Key_Escape) {
        this->close();
        emit keyPressed(event);
        return;
    }
    if (event->key() == Qt::Key_F11) {this->switchFullScreen();return;}
    if (event->key() == Qt::Key_Space && (kind & CONTAINER_KIND_VIDEO)) {this->switchPlayPause();return;}
    emit keyPressed(event);
}

bool mediaPlayer::event(QEvent *e)
{
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO) {
        switch(e->type())
        {
        case QEvent::MouseMove:
            if (!isFullscreen) break;
#ifndef QT_NO_CURSOR
            unsetCursor();
#endif
            for (int i = 0; i < horizontalLayout->count(); ++i)
                horizontalLayout->itemAt(i)->widget()->show();
            if (adjustVideoWidget!=0) adjustVideoWidget->show();
            if (volumeSlider!=0)      volumeSlider->show();
        case QEvent::WindowStateChange:
            {
                if (windowState() & Qt::WindowFullScreen) {
                    timer.start(1500, this);
                } else {
                    timer.stop();
                    for (int i = 0; i < horizontalLayout->count(); ++i)
                        horizontalLayout->itemAt(i)->widget()->show();
                    if (adjustVideoWidget!=0) adjustVideoWidget->show();
                    if (volumeSlider!=0)      volumeSlider->show();
#ifndef QT_NO_CURSOR
                    unsetCursor();
#endif
                }
            }
            break;
        case QEvent::MouseButtonDblClick:
            if (windowState() & Qt::WindowFullScreen) goNormalScreen();
            else goFullScreen();
            break;
        default:
            break;
        }
    }
#endif
    //    return Phonon::VideoWidget::event(e);
    return QWidget::event(e);
}


void mediaPlayer::timerEvent(QTimerEvent *e)
{
#ifdef PHONON_LIB
    if (e->timerId() == timer.timerId()) {
        if (adjustVideoWidget==0 && volumeSlider==0) {
            for (int i = 0; i < horizontalLayout->count(); ++i)
                horizontalLayout->itemAt(i)->widget()->hide();
            if (adjustVideoWidget!=0) adjustVideoWidget->hide();
            if (volumeSlider!=0)      volumeSlider->hide();
#ifndef QT_NO_CURSOR
            setCursor(Qt::BlankCursor);
#endif
        }
    }
#endif
    //    Phonon::VideoWidget::timerEvent(e);
    QWidget::timerEvent(e);
}


//void mediaPlayer::onKeyPressed( QKeyEvent * event )
//{
//    keyPressEvent(event);
////    emit keyPressed(event);
//}

void mediaPlayer::setGeometry( QRect new_rect )
{
    if (resizing) return;

    new_rect.translate(new_rect.topLeft()*(-1));

//#if QT_VERSION > 0x040600
//    QRect old_rect=this->geometry();
//    if (old_rect.size()==QSize(0,0))
//        old_rect=QRect(parentWidget->width()/2, parentWidget->height()/2, 0,0);

//    zoom->setStartValue( old_rect );
//    zoom->setDuration(300);
//    zoom->setEndValue( new_rect );
//    zoom->start();
//    resizing=true;
//#else
//    QWidget::setGeometry(new_rect);
//#endif
    QWidget::setGeometry(new_rect);
    this->show();
}

void mediaPlayer::setGeometry( int x, int y, int w, int h )
{
    setGeometry(QRect(x,y,w,h));
}

void mediaPlayer::resizingFinished()
{
    resizing=false;
}

void mediaPlayer::goFullScreen()
{
    if (isFullscreen) return;
    this->setParent(0);
    this->setWindowFlags(Qt::FramelessWindowHint | Qt::Widget | Qt::WindowStaysOnTopHint);
    this->setStyleSheet(parentWidget->styleSheet());
    this->setGeometry(QApplication::desktop()->screenGeometry(-1));
    this->showFullScreen();
#ifdef PHONON_LIB
    if (!videoPlayer.isNull()) videoPlayer->setMouseTracking(true);
#endif
    this->setMouseTracking(true);

    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
    this->activateWindow();
    isFullscreen=true;
    emit stateChanged();
}

void mediaPlayer::goNormalScreen()
{
    this->setWindowFlags(Qt::FramelessWindowHint | Qt::Widget | Qt::WindowStaysOnTopHint);
    this->setParent(parentWidget);
    this->setGeometry(parentWidget->geometry());
    this->showNormal();

    this->show();
    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
    this->activateWindow();
    isFullscreen=false;
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO) {
        timer.stop();
        for (int i = 0; i < horizontalLayout->count(); ++i)
            horizontalLayout->itemAt(i)->widget()->show();
        if (adjustVideoWidget!=0) adjustVideoWidget->show();
        if (volumeSlider!=0)      volumeSlider->show();
#ifndef QT_NO_CURSOR
        unsetCursor();
#endif
    }
#endif
    emit stateChanged();
}


void mediaPlayer::switchPlayPause()
{
    if (kind & CONTAINER_KIND_VIDEO) {
#ifdef PHONON_LIB
        if (videoPlayer->isPlaying()) {
            PB_playPause->setIcon(PB_playPause->style()->standardIcon(QStyle::SP_MediaPlay));
            videoPlayer->pause();
        } else
        {
            PB_playPause->setIcon(PB_playPause->style()->standardIcon(QStyle::SP_MediaPause));
            videoPlayer->play();
        }
#endif
    }
}

void mediaPlayer::switchFullScreen()
{
    QIcon icon;
    if (isFullscreen) {
        goNormalScreen();
        icon.addFile(QString::fromUtf8(":/icons/icons/fullscreen.png"), QSize(), QIcon::Normal, QIcon::Off);
    }
    else {
        goFullScreen();
        if (kind & CONTAINER_KIND_VIDEO) icon.addFile(QString::fromUtf8(":/icons/icons/nofullscreen.png"), QSize(), QIcon::Normal, QIcon::Off);
    }
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO) PB_fullscreen->setIcon(icon);
#endif
    // Set pixmap in the new format
    if (kind & CONTAINER_KIND_IMAGE) this->play(currPix);
}

// Size is the size of the container for the pixmap
void mediaPlayer::play(QPixmap &pixmap)
{
    QRect size;
    if (pixContainer.isNull()) this->setupUi();

    currPix = pixmap;
    if (isFullscreen) {
        size=QApplication::desktop()->screenGeometry(-1);
        goFullScreen();
    } else {
        size=parentWidget->geometry();
        goNormalScreen();
    }

    pixContainer->setGeometry(size);
    pixContainer->setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgb(38, 38, 194), stop:0.506726 rgb(0, 0, 0), stop:1 rgb(38, 38, 194));");
    pixContainer->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    pixContainer->setPixmap(pixmap.scaled(size.size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
    pixContainer->setFocus();
    pixContainer->activateWindow();
}

#ifdef PHONON_LIB
// Check if phonon backend can play mp4 and flv
void mediaPlayer::checkPhononCapabilities()
{
    QStringList sl=Phonon::BackendCapabilities::availableMimeTypes();

    phonon_capabilities.flv=sl.contains("video/x-flv") || sl.contains("video/x-flash-screen") ||
                sl.contains("video/x-flash-video") || sl.contains("video/quicktime");
    phonon_capabilities.mp4=sl.contains("video/x-h264") ||
                sl.contains("video/mp4") || sl.contains("video/mpg");
}

// Generic play with a given list of videos. Defaults play 1st video
void mediaPlayer::play(YouTubeStruct t)
{
    if (t.video_info.size()==0) return;
    kind=CONTAINER_KIND_VIDEO;
    if (videoPlayer.isNull()) this->setupUi();
    curr_tube=t;
    CB_formats->disconnect(SIGNAL(currentIndexChanged(int)) , this);
    CB_formats->clear();
    CB_formats->setCurrentIndex(0);
    for (int i=0; i< t.video_info.size(); i++)
        CB_formats->addItem(t.video_info.at(i).fmt+" - "+t.video_info.at(i).humanFmt+" - "+t.video_info.at(i).container);
    CB_formats->setCurrentIndex(t.ID_preferred_video_format);
    connect(CB_formats, SIGNAL(currentIndexChanged(int)) , this, SLOT(play(int)));
    play( t.ID_preferred_video_format );
}

void mediaPlayer::play(int i)
{
    if (i>curr_tube.video_info.size()) return;
    curr_playing_video=i;

    play(curr_tube.video_info.at(i).url_play);
}

// Play specific video url
void mediaPlayer::play(QString url)
{
    if (videoPlayer.isNull()) this->setupUi();
    if (videoPlayer->isPlaying()) {
        videoPlayer->stop();
    }
    videoPlayer->update();
    videoPlayer->repaint();
    // Withouot opengl seems that the 1st time playing video, the widget
    // isn't updated. Pressing fullscreen and restore works
#ifndef QT_OPENGL_SUPPORT
    if (isFullscreen) goNormalScreen();
    else goFullScreen();
    if (isFullscreen) goNormalScreen();
    else goFullScreen();
#endif

    qDebug() << "Playing: " << url;

    seekSlider->setMediaObject(mediaObject);

    this->show();
    videoPlayer->play(url);

    this->activateWindow();
    this->setFocus();
}

bool mediaPlayer::isPlaying()
{
    return videoPlayer->isPlaying();
}

void mediaPlayer::stopPlay()
{
    ticks=0;
    videoPlayer->stop();
}

void mediaPlayer::bufferChanged(int i)
{
    qDebug() << "Buffer: " << i;
}

void mediaPlayer::seekableChanged(bool b)
{
    Q_UNUSED(b);

    qDebug() << "Seekable: " << b;
    qDebug() << "HasTracking: " << seekSlider->hasTracking();
    seekSlider->setTracking(false);
    qDebug() << "HasTracking: " << seekSlider->hasTracking();
}

void mediaPlayer::tick(qint64 time)
{
    if (time <= 0) return;
    ticks=mediaObject->remainingTime();
    L_time->setText(QString::number(ticks/1000)+"/"+QString::number(mediaObject->totalTime()/1000));
}

void mediaPlayer::popupInfo()
{
    QString s;
    s="<font size=\"4\"><p>"+curr_tube.duration+"\" "+
      curr_tube.video_info.at(curr_playing_video).container+" - "+curr_tube.video_info.at(curr_playing_video).humanFmt+"</p>"+
      "<p>"+curr_tube.published+" by "+curr_tube.author_name+"</p>"+
      "<p>Viewed: "+curr_tube.statistics_viewCount+" Rating: "+curr_tube.rating_average+" with "+curr_tube.rating_numRaters+" votes </p>"+
      "<p><h3><strong>"+curr_tube.title+"</strong></h3></p>"+
      curr_tube.content+
      "</font>";

      popupText *p=new popupText(this);
      p->showHtmlText(s, QRect(this->PB_info->pos().x(), this->PB_info->y()/*+this->PB_info->height()*/,
                               this->width()/3*2, this->height()/3*2 )
                      );
}

void mediaPlayer::onVideoPlayFinished()
{
    ticks=0;
    emit videoPlayFinished();
}

#endif


