/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QMessageBox>
#include <QDebug>
#include <QDomDocument>
#include <QDesktopServices>
#include "onlinecheck.h"
#include "settings.h"
#include "defines.h"
#include "ui_onlinecheck.h"

onlineCheck::onlineCheck(bool force_check, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::onlineCheck)
{
    QDate lastDayChecked;
    settings conf;
    today=QDate::currentDate();
    lastDayChecked=QDate::fromJulianDay(
            conf.conf->value("main/lastOnlineCheck",
                             QDate::currentDate().addDays(-30)).toInt()
            );
    if (!force_check && today.toJulianDay()-lastDayChecked.toJulianDay() < 30) {
        this->close();
        return;
    }
    ui->setupUi(this);
    ui->webView->page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
    connect(ui->webView, SIGNAL(linkClicked(QUrl)), this, SLOT(linkDelegate(QUrl)));
    this->setAttribute(Qt::WA_DeleteOnClose);

    this->check();
}

onlineCheck::~onlineCheck()
{
    delete ui;
}

void onlineCheck::linkDelegate(QUrl url)
{
    QDesktopServices::openUrl(url);
}

void onlineCheck::check()
{
    settings conf;

    if (QMessageBox::question(this,
                              tr("Online check"),
                              tr("Do you want to check online updates/news ?\nWill ask again in 30 days"),
                              QMessageBox::Ok | QMessageBox::Cancel)
        == QMessageBox::Cancel) {
        this->close();
        conf.store_settings("main/lastOnlineCheck", today.toJulianDay());
        return;
    }
    QByteArray buffer;
    bool b;
    net=new network(false, this);
    b=net->outDebug;
    net->outDebug=false;
    buffer=net->get_synchronous(CHECK_UPDATE_URL, false);
    net->outDebug=b;
//    qDebug() << net->net[0].error_number;
//    if (net->net[0].error_number==QNetworkReply::NoError)
    if (!buffer.isEmpty()) parseXML(&buffer);
    else {
        QString s;
        s=tr("Problems retrieving data.<br />");
        s+=tr("Try to see online at ");
        s+="<a href=\"http://mediadownloader.cz.cc\">mediadownloader.cz.cc</a><br />";
        s+="<a href=\"http://sourceforge.net/projects/googleimagedown/files/\">SourcefForge</a>";
        ui->webView->setHtml(s);
    }

    this->show();
    conf.store_settings("main/lastOnlineCheck", today.toJulianDay());
}

void onlineCheck::parseXML(QByteArray *b)
{
    QString currVersion;
    QString currVersionRevision;
    QString latestPublished;
    QString latestPublishedNotes;
    QString latestRevisionVersion;
    QString latestRevisionStatus;
    QString latestRevisionNotes;
    QString generalNotes;

    currVersion=VERSION_NUMBER;
    currVersionRevision=VERSION_REVISION;
    QDomDocument doc("mediadownloader");
    doc.setContent(*b);

    QDomNode n = doc.firstChild();
    while (!n.isNull()) {
        if (n.toElement().tagName()=="html") n = n.firstChild();
        if (n.toElement().tagName()=="head") n = n.nextSibling();
        if (n.toElement().tagName()=="body") n = n.firstChild();
        if (n.toElement().tagName()=="mediadownloader") {
//            qDebug() << "Element name: " << n.toElement().tagName() << endl;
            latestPublished       = n.toElement().namedItem("latestPublished").toElement().text();
            latestPublishedNotes  = n.toElement().namedItem("latestPublishedNotes").toElement().text();
            latestRevisionVersion = n.toElement().namedItem("latestRevisionVersion").toElement().text();
            latestRevisionStatus  = n.toElement().namedItem("latestRevisionStatus").toElement().text();
            latestRevisionNotes   = n.toElement().namedItem("latestRevisionNotes").toElement().text();
            generalNotes          = n.toElement().namedItem("generalNotes").toElement().text();
            break;
        }
        n = n.nextSibling();
    }


    QString s;
    int i;
//    i=QString::compare("1.4.1","1.4", Qt::CaseSensitive); 2
//    i=QString::compare("1.4","1.4.1", Qt::CaseSensitive); -2
//    i=QString::compare("1.4.1","1.4.1", Qt::CaseSensitive); 0
//    i=QString::compare("1.3.2","1.4.1", Qt::CaseSensitive); -1

    // Comparing installed version with the latest online
    // If there is a beta (latestRevisionVersion set > current), out the description
    i=QString::compare(currVersion,latestPublished);
    if (i==0) {
        s="You are using the latest version";
    }
    if (i<0) {
        s ="<strong>Your current version is: </strong>"+currVersion+" "+currVersionRevision+"<br />";
        s+="Version "+latestPublished+" is available.<br />";
        s+=latestPublishedNotes;
    }
    if (i>0) {
        s="You are using the latest version: "+latestRevisionVersion+" "+latestRevisionStatus;
    }
    // If present, write availability of beta
    if (QString::compare(currVersion,latestRevisionVersion)<0)
        s+="<br /><br />"+latestRevisionNotes;

    if (!generalNotes.isEmpty()) s+="<br /><br />"+generalNotes;

    ui->webView->setHtml(s);
    ui->webView->adjustSize();
}
