/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QDebug>
#include <QFileInfo>
#include <QMessageBox>
#include "mainwindow.h"
#include "save.h"
#include "imagecontainer.h"
#include "googlepicture.h"
#include "youtube.h"

network *save::net_async=0;

save::save(QDialog *parent)
{
    ffmpegDialog=dynamic_cast<ffmpegFrontend*>(parent);
}

QString save::checkFileName(QString s)
{
     s.replace("\\","");
     s.replace("/","");
     s.replace(":","");
     s.replace("*","");
     s.replace("?","");
     s.replace("\"","");
     s.replace("<","");
     s.replace(">","");
     s.replace("|","");
     s.replace("&","");
     s.replace("%","");
     return s;
}

void save::save_to_local_file(QByteArray buffer,
                              imageStruct item,
                              int n,
                              QString default_ext)
{
    QFile *f;
    QString file_name;
    QString url;
    QString tmp;
    QFileInfo fileInfo;
    bool isVideoFile=false;
    bool fileNameFound=false;

    while (!fileNameFound && n<=9999)
    {
        // make the complete file path name
        tmp="0000";
        // Check if downloading url is a youtube url so I can use the &id tag for filename
        QString videoFileName;
        if (item.kind & CONTAINER_KIND_VIDEO) {
            isVideoFile=true;
            if (item.kind == CONTAINER_KIND_VIDEO_OTHER)
                videoFileName=QString::number(item.ID);
            else
                videoFileName=item.youTubeInfo.title;
        }

        if (item.kind & CONTAINER_KIND_IMAGE) {
            url=item.googleImageInfo.url;
        }

        if (options_for_save==USE_NUMBER || options_for_save==USE_NUMBER_PLUS_FILENAME) {
            file_name=QString::number(1+n+start_image_number);
            file_name.prepend(tmp);
            file_name.remove(0,file_name.length()-4);
        }
        if (options_for_save==USE_NUMBER_PLUS_FILENAME) {
            fileInfo.setFile( url );
            if (isVideoFile) file_name=file_name+"-"+videoFileName;
            else   file_name=file_name+"-"+fileInfo.fileName();
        }
        if (options_for_save==USE_FILENAME) {
            if (isVideoFile)
                file_name=videoFileName;
            else
            {
                fileInfo.setFile( QUrl(url).path());
                file_name= fileInfo.fileName();
            }
        }

        file_name = checkFileName(file_name);
        fileInfo.setFile(file_name);
        if (fileInfo.suffix().toLower().size()==0) file_name+=default_ext;
        file_name = QUrl::fromPercentEncoding( QUrl(file_name).path().toAscii());
        file_name = dir_to_save+"/"+file_name;
        fileInfo.setFile(file_name);
        if ( fileInfo.exists() )
            n++;
        else
            fileNameFound=true;
    }

    f=new QFile(file_name);
    if (!f->open(QIODevice::WriteOnly))
    { qDebug() << "error writing file " << f->fileName(); }
    f->write(buffer);
    f->close();
    if (convert_video && isVideoFile) {
        QStringList a=ffmpegDialog->getArgs();
        ffmpegDialog->runFfmpeg(ffmpegDialog->getArgs(),
                                QFileInfo(*f));
    }
    delete f;
}

void save::download_one(imageStruct item,
                        QString local_dir,
                        save_options opt,
                        int start,
                        bool convert)
{
    QByteArray buffer;

    dir_to_save=local_dir;
    options_for_save=opt;
    start_image_number=start;
    convert_video=convert;


    if (item.kind & CONTAINER_KIND_IMAGE)
    {
        net=new network(true, MainWindow::pointerToMainWindow);
        buffer=net->get_synchronous(item.googleImageInfo.url,false);
        if (!buffer.isEmpty())
            save_to_local_file(buffer, item, 0, ".jpg");
        net->deleteLater();
    }
    if (item.kind == CONTAINER_KIND_VIDEO)
    {
        if (item.youTubeInfo.video_info.size()==0)
            youTube::getTubeVideoInfo(item.youTubeInfo);

        // Messagebox to ask which video res is to download
        QMessageBox *msgBox = new QMessageBox(MainWindow::pointerToMainWindow);
        msgBox->setText(tr("Which resolution do you want to download?"));
        QVBoxLayout *layout = new QVBoxLayout;
        msgBox->setLayout(layout);
        for (int i=0; i<item.youTubeInfo.video_info.size(); i++)
            msgBox->addButton(item.youTubeInfo.video_info.at(i).humanFmt + " - " +
                             item.youTubeInfo.video_info.at(i).container ,
                             QMessageBox::ActionRole);
        msgBox->addButton(QMessageBox::Cancel);//"Cancel",QMessageBox::RejectRole);
        int ret = msgBox->exec();
        delete msgBox;

        if (ret!=QMessageBox::Cancel) {
            if (item.youTubeInfo.video_info.size() > 0) {
                net=new network(true, MainWindow::pointerToMainWindow);
                buffer=net->get_synchronous(item.youTubeInfo.video_info.at(ret).url_retrieve, false);
                delete net;
        }
        else
            qDebug() << "Error retrieving iformations for video id: " << item.youTubeInfo.video_id;
        if (!buffer.isEmpty())
            save_to_local_file(buffer, item, 0, "."+item.youTubeInfo.video_info.at(ret).container);
        }
    }
    if (item.kind == CONTAINER_KIND_VIDEO_OTHER)
    {
        net=new network(true, MainWindow::pointerToMainWindow);
        buffer=net->get_synchronous(item.image_url, false);
        delete net;
        if (!buffer.isEmpty()) {
            QString ext="";
            if (item.description_row2.contains("flv"))       ext=".flv";
            if (item.description_row2.contains("mp4"))       ext=".mp4";
            if (item.description_row2.contains("mpeg"))      ext=".mpg";
            if (item.description_row2.contains("msvideo"))   ext=".avi";
            if (item.description_row2.contains("quicktime")) ext=".mov";
            save_to_local_file(buffer, item, 0, ext);
        }
    }
    emit downloadsComplete();
}

void save::download_all(QList<imageStruct> items,
                        QString local_dir,
                        save_options opt,
                        int start,
                        bool convert)
{
    if (items.size()==0) return;
    imageStruct i;
//    QList<youTubeVideoInfo> video_available;
    int ID;

    items_to_download=items;
    dir_to_save=local_dir;
    options_for_save=opt;
    start_image_number=start;
    convert_video=convert;

    net=new network(true, MainWindow::pointerToMainWindow);

    connect(net, SIGNAL(downloadFinished(int,QByteArray)),
            this, SLOT(downloaded(int,QByteArray)));
    connect(net, SIGNAL(queue_completed()),
            this, SLOT(allDownloaded()));
    connect(net, SIGNAL(destroyed()),
            this, SLOT(allDownloaded()) );

    default_extensions.clear();
    foreach (i,items)
    {
        if (i.kind==CONTAINER_KIND_IMAGE)
        {
            net->get(i.googleImageInfo.url);
            default_extensions.append(".jpg");
        }
        if (i.kind==CONTAINER_KIND_VIDEO)
        {
            youTube::getTubeVideoInfo(i.youTubeInfo);
            ID=i.youTubeInfo.ID_preferred_video_format;
            if (i.youTubeInfo.video_info.size()>0)
            {
                default_extensions.append("."+i.youTubeInfo.video_info.at(ID).container);
                net->get(i.youTubeInfo.video_info.at(ID).url_retrieve);
            } else
                qDebug() << "Error retrieving iformations for video id: " << i.youTubeInfo.video_id;
        }
    }
}


void save::downloaded(int n, QByteArray buffer)
{
    if (buffer.isEmpty()) return;

    save_to_local_file(buffer,
                       items_to_download.at(n),
                       n,
                       default_extensions[n]);
}

void save::allDownloaded()
{
    net->deleteLater();
    emit downloadsComplete();
}
