/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef SLIDESHOW_H
#define SLIDESHOW_H

#include <QDialog>

#include <QTimer>
#include <QPointer>
#include "imagewidget.h"
#include "imagecontainer.h"
#include "mediaplayer.h"
#include "network.h"

namespace Ui {
    class slideshow;
}


class slideshow : public QDialog
{
    Q_OBJECT

public:
    explicit slideshow(QList<imageContainer *> itemsList,
                       int _delay=0,
                       bool _fullScreen=true,
                       bool _loop=true,
                       bool _showAll=true,
                       bool _showImages=false,
                       bool _showVideos=false,
                       QWidget *parent = 0);
    ~slideshow();

    void backwardCurrItem();
    void forwardCurrItem();
    void loadCurrItem();

    int delay;
    bool fullScreen;
    bool loop;
    bool showAll;
    bool showImages;
    bool showVideos;

private:
    QList<imageContainer *> items;
    QPointer<mediaPlayer> player;
    network *net;
    QTimer *timer;
    int curr_item_index;
    int timer_delay;
    QPixmap pixmap;
    QPointer<progressWidget> progress;
    bool imageLoaded;
    bool first_slide_started;
    bool noMoreItems;


protected:
    virtual void keyPressEvent( QKeyEvent * event );

private slots:
    void onKeyPressed( QKeyEvent * event );
    void displayCurrItem();
    void downloadProgress(int i, qint64 current, qint64 total);
    void downloaded(int n, QByteArray buffer);
    void downloadError(int i, QNetworkReply::NetworkError e);
    void playNextItem();
    void playPreviousItem();

};

#endif // SLIDESHOW_H
