/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QMainWindow>
#include <QByteArray>
#include <QtGlobal>
#include <QDebug>
#include <QProcess>

#include "defines.h"
#include "styles.h"



// If we are in debugging mode, the css are searched where .pro file is stored in css dir
// If release mode:
// in linux css are copied from /etc/mediadownloader/css (created during package install) to ~/.mediadownloader/css
// in windows are in css directory under the program executable

QDir styles::CSSpath=QDir("");

styles::styles(QString fileName, QObject *parent) :
    QObject(parent)
{
    styleFile = new QFileInfo("fileName");
    setInitialSettings();
    getCssFileList();
    if (fileName.contains(tr("+watch"))) {
        fileName = fileName.replace(tr("+watch"),"");
        setStyle(fileName , true);
    }
    else
        setStyle(fileName , false);
}

styles::~styles()
{
    if (!cssFileWatcher.isNull()) delete cssFileWatcher;
    delete styleFile;
}

// watch will monitor file changes in realtime
void styles::setStyle(QString fileName, bool watch)
{
    if (fileName==tr("System style"))
        qobject_cast<QMainWindow*>(this->parent())->setStyleSheet("");

    if (fileName.contains(tr("+watch"))) {
        fileName = fileName.replace(tr("+watch"),"");
        watch=true;
    }

    styleFile->setFile(CSSpath.absolutePath(),QString(fileName+".css"));

    QFile f(styleFile->absoluteFilePath());
    if (!f.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QByteArray content;
    content = f.readAll();
    if (!content.isEmpty())
        qobject_cast<QMainWindow*>(this->parent())->setStyleSheet(content);

    if (watch) {
        // Monitor file changes
        if (!cssFileWatcher.isNull()) delete cssFileWatcher;
        cssFileWatcher = new QFileSystemWatcher(this);
        qDebug() << "Watching: " << styleFile->absoluteFilePath();
        cssFileWatcher->addPath(styleFile->absoluteFilePath());
        connect(cssFileWatcher, SIGNAL(fileChanged(QString)),
                this, SLOT(cssFileChanged(QString)) );
    }
}

void styles::cssFileChanged(QString s)
{
    qDebug () << s << " changed";
    setStyle(styleFile->baseName(),true);
}

// Initial setting for CSSpath.
void styles::setInitialSettings()
{
#ifdef DEBUGGING
    CSSpath.setPath(QString(QUOTEME(PWD_PRO))+"/css");
#else
    #ifdef Q_WS_X11
        CSSpath.setPath(QDir::homePath()+"/.mediadownloader/css");
    #endif

    #if defined(Q_WS_WIN) || defined(Q_WS_PM)
        CSSpath.setPath(QDir::currentPath()+"/css");
    #endif
#endif
    if (CSSpath.exists()) qDebug() << "css dir: " << CSSpath.absolutePath();
}

QFileInfoList styles::getCssFileList()
{
    if (CSSpath==QDir("")) {
        styles::setInitialSettings();
//        qDebug() << CSSpath.absolutePath() << " will be used for styles";
    }
    CSSpath=CSSpath.toNativeSeparators(CSSpath.absolutePath());
    return CSSpath.entryInfoList(QStringList("*.css"),QDir::Files,QDir::Name);
}
