/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef YOUTUBE_H
#define YOUTUBE_H

#include <QObject>
#include <QString>

typedef struct {
    int fmt;
    QString res;
    QString container;
    long area;
} video_formats;

typedef struct {
    qint16 height;
    qint16 width;
    QString url;
} youTubeThumb;

typedef struct {
    QString fmt;      // string to append to url like url+="&fmt="+fmt;
    QString humanFmt; // display format like 480p 720p 1280p etc.
    int xres,yres;
    QString container;     // flv mkv 3gp etc
    QString url_play;      // url for play this video
    QString url_retrieve;  // url for download this video
} youTubeVideoInfo;

typedef struct {
    QString id;
    QString video_id;
    QString published;
    QString updated;
    QString title;
    QString content;
    QString author_name;
    QString author_uri;
    youTubeThumb p[4];  // Thumbnails for the video, usually [3] is 320x200 other 170x140 or similar
    QString mediaUrl;
    QString duration;
    QString rating_average;
    QString rating_numRaters;
    QString statistics_viewCount;
    QList<youTubeVideoInfo> video_info;
    quint8 ID_preferred_video_format;
} YouTubeStruct;


enum StandardVideoFeeds {
    VIDEO_FEED_NONE_USE_SEARCH,
    VIDEO_FEED_TOP_RATED,
    VIDEO_FEED_TOP_FAVORITES,
    VIDEO_FEED_MOST_VIEWED,
    VIDEO_FEED_MOST_POPULAR,
    VIDEO_FEED_MOST_RECENT
};

QString getUrlsFromPattern(int url_or_strem, QByteArray data, int itag);
QString unescapeCharacters(QString s);

class youTube {
    youTube();

public:
    static const video_formats videoFormats[];
    static int ID_preferred_video_format;
    static bool only_mp4;
    static int getTubeVideoInfo(YouTubeStruct &tube);
    static int getPreferredResolution(YouTubeStruct &tube, int ID);
    static QList<YouTubeStruct> fillSearchTubes(QString getUrl);
    static QList<YouTubeStruct> getList(QString search,
                                 StandardVideoFeeds=VIDEO_FEED_NONE_USE_SEARCH,
                                 int start_feed_number=1, int feeds_to_get=20,
                                 int order=0,
                                 QString preferred_language="en",
                                 int time=0
                                          );

};

#endif // YOUTUBE_H
